#ifndef __COFF_PIC30_H__
#define __COFF_PIC30_H__

extern int pic30_adjustable_against_section (unsigned short);

#define PIC30_BITS_PER_PROGRAM_ADDRESS 24

#define PIC30_TBLOFFSET(value) ((value) & 0xFFFF)
#define PIC30_PSVOFFSET(value) (((value) | 0x8000) & 0xFFFF)
#define PIC30_TBLPAGE(value)  ((value) >> 16)
#define PIC30_PSVPAGE(value)  ((value) >> 15)
#define PIC30_HANDLE(value) ((value) & 0xFFFF)

#define PIC30_DATA_WINDOW_BASE 0x8000

#define PIC30_IS_EVEN(value) (((value) & 0x1) == 0)
#define PIC30_IS_VALID_DO_OFFSET(lit) \
        (((lit) >= -32768) && ((lit) < 32768) && \
         ((lit) != -1) && ((lit) != 0) && ((lit) != 1))
#define PIC30_IS_1_BIT_LITERAL(lit) (((lit) >= 0) && ((lit) < 2))
#define PIC30_IS_3_BIT_LITERAL(lit) (((lit) >= 0) && ((lit) < 8))
#define PIC30_IS_4_BIT_LITERAL(lit) (((lit) >= -8) && ((lit) < 8))
#define PIC30_IS_4_BIT_UNSIGNED_LITERAL(lit) (((lit) >= 0) && ((lit) < 16))
#define PIC30_IS_5_BIT_LITERAL(lit) (((lit) >= -16) && ((lit) < 16))
#define PIC30_IS_5_BIT_UNSIGNED_LITERAL(lit) (((lit) >= 0) && ((lit) < 32))
#define PIC30_IS_DSP6_BIT_LITERAL(lit) (((lit) >= -16) && ((lit) <= 16))
#define PIC30_IS_BYTE_VALUE(lit) (((lit) >= -128) && ((lit) < 256))
#define PIC30_IS_10_BIT_UNSIGNED_LITERAL(lit) ((0 <= (lit)) && ((lit) <= 1023))
#define PIC30_IS_14_BIT_UNSIGNED_LITERAL(lit) (((lit) >= 0) && ((lit) < 16384))
#define PIC30_IS_16_BIT_LITERAL(lit) (((lit) >= -32768) && ((lit) < 65536))
#define PIC30_IS_16_BIT_UNSIGNED_LITERAL(lit) (((lit) >= 0) && ((lit) < 65536))
#define PIC30_IS_13_BIT_FILE_REG(lit) (((lit) >= 0) && ((lit) < 8192))

#define FILE_REG_LIMIT 8192

/* An Entry in the Handle Hash Table */
struct handle_hash_entry
{
  struct bfd_hash_entry root;
  int index;     /* jump table index    */
  char *sym;     /* name of jump target */
};

/* The Handle Hash table */
struct handle_hash_table
{
  struct bfd_hash_table table;
  int num;  /* # of entries */
};

/* Look Up an Entry in the Handle Hash Table */
#define handle_hash_lookup(t, string, create, copy) \
  ((struct handle_hash_entry *) \
   bfd_hash_lookup (&(t)->table, (string), (create), (copy)))

#define PIC30_SECTION_IN_PROGRAM_MEMORY(sec) \
  ((sec->flags & SEC_CODE))
#define PIC30_SECTION_IN_DATA_MEMORY(sec) \
  ((sec->flags & SEC_READONLY) || (sec->flags & SEC_DATA))

/*
** The following macros are used by
** objdump and the disassembler.
*/

/* CODE section */
#define PIC30_DISPLAY_AS_PROGRAM_MEMORY(sec) \
   ((sec->flags & SEC_CODE))

/* linked READONLY section */
#define PIC30_DISPLAY_AS_READONLY_MEMORY(sec) \
  ((sec->flags & SEC_READONLY) && (sec->lma))

/* BSS, DATA, or unlinked READONLY section */ 
#define PIC30_DISPLAY_AS_DATA_MEMORY(sec) \
  (((sec->flags & (SEC_CODE | SEC_READONLY)) == 0) || \
   ((sec->flags & SEC_READONLY) && (sec->lma == 0)))

#define PIC30_CAN_PRINT_IN_PROGRAM_MEMORY(addr) \
  ((addr % 4) != 3)

#define PIC30_CAN_PRINT_IN_READONLY_MEMORY(addr) \
  ((addr % 4) < 2)

#define PIC30_CAN_PRINT_IN_DATA_MEMORY(addr) \
  ((addr % 2) != 1)


#endif

