/*
 * cpu-pic30.c -- BFD library support routines for the PIC30 architecture.
 * Copyright (C) 2000 Free Software Foundation, Inc.
 * Contributed by Microchip Corporation.
 * Written by Tracy A. Kuhrt
 *
 * This file is part of BFD, the Binary File Descriptor library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "bfd.h"
#include "sysdep.h"
#include "libbfd.h"

/******************************************************************************/

const bfd_arch_info_type * global_PROCESSOR = NULL;

#define ARCH(machine, printable_name, default_machine, next) \
{ 16,                     \
  24,                     \
  16,                      \
  bfd_arch_pic30,         \
  machine,                \
  "pic30",                \
  printable_name,         \
  1,                      \
  default_machine,        \
  bfd_default_compatible, \
  bfd_default_scan,       \
  next }

static const bfd_arch_info_type arch_info_struct[] =
{
   ARCH ( bfd_mach_P30F2010, "30F2010", false, &arch_info_struct[1] ),
   ARCH ( bfd_mach_P30F2011, "30F2011", false, &arch_info_struct[2] ),
   ARCH ( bfd_mach_P30F2012, "30F2012", false, &arch_info_struct[3] ),
   ARCH ( bfd_mach_P30F3010, "30F3010", false, &arch_info_struct[4] ),
   ARCH ( bfd_mach_P30F3011, "30F3011", false, &arch_info_struct[5] ),
   ARCH ( bfd_mach_P30F3012, "30F3012", false, &arch_info_struct[6] ),
   ARCH ( bfd_mach_P30F3013, "30F3013", false, &arch_info_struct[7] ),
   ARCH ( bfd_mach_P30F3014, "30F3014", false, &arch_info_struct[8] ),
   ARCH ( bfd_mach_P30F5015, "30F5015", false, &arch_info_struct[9] ),
   ARCH ( bfd_mach_P30F4011, "30F4011", false, &arch_info_struct[10] ),
   ARCH ( bfd_mach_P30F4012, "30F4012", false, &arch_info_struct[11] ),
   ARCH ( bfd_mach_P30F4013, "30F4013", false, &arch_info_struct[12] ),
   ARCH ( bfd_mach_P30F4014, "30F4014", false, &arch_info_struct[13] ),
   ARCH ( bfd_mach_P30F4015, "30F4015", false, &arch_info_struct[14] ),
   ARCH ( bfd_mach_P30F5010, "30F5010", false, &arch_info_struct[15] ),
   ARCH ( bfd_mach_P30F5011, "30F5011", false, &arch_info_struct[16] ),
   ARCH ( bfd_mach_P30F5012, "30F5012", false, &arch_info_struct[17] ),
   ARCH ( bfd_mach_P30F5013, "30F5013", false, &arch_info_struct[18] ),
   ARCH ( bfd_mach_P30F5014, "30F5014", false, &arch_info_struct[19] ),
   ARCH ( bfd_mach_P30F6010, "30F6010", false, &arch_info_struct[20] ),
   ARCH ( bfd_mach_P30F6011, "30F6011", false, &arch_info_struct[21] ),
   ARCH ( bfd_mach_P30F6012, "30F6012", false, &arch_info_struct[22] ),
   ARCH ( bfd_mach_P30F6013, "30F6013", false, &arch_info_struct[23] ),
   ARCH ( bfd_mach_P30F6014, "30F6014", false, &arch_info_struct[24] ),
   ARCH ( bfd_mach_P30F6010, "30F6010A2", false, &arch_info_struct[25] ),
   ARCH ( bfd_mach_P30F6014, "30F6014A2", false, NULL )
};

const bfd_arch_info_type bfd_pic30_arch = 
   ARCH ( 0,                 "30Fxxxx", true,  &arch_info_struct[0] );

const bfd_arch_info_type *
pic30_get_machine (name)
   const char * const name;
{
   const bfd_arch_info_type * p;

   for (p = bfd_pic30_arch.next; p != NULL; p = p->next)
   {
      if (strcasecmp (name, p->printable_name) == 0)
         break;
   } /* for p */

   return p;
} /* const bfd_arch_info_type * pic30_get_machine(const char * const) */
