# This shell script emits a C file. -*- C -*-
# It does some substitutions.
cat >e${EMULATION_NAME}.c <<EOF
/* This file is is generated by a shell script.  DO NOT EDIT! */

/* emulate the original gld for the given ${EMULATION_NAME}
   Copyright (C) 1991, 93, 94, 95, 96, 1999 Free Software Foundation, Inc.
   Written by Steve Chamberlain steve@cygnus.com

This file is part of GLD, the Gnu Linker.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

#define TARGET_IS_${EMULATION_NAME}

#include "bfd.h"
#include "sysdep.h"
#include "bfdlink.h"
#include "getopt.h"
#include "genlink.h"

#include "ld.h"
#include "ldmain.h"
#include "ldmisc.h"

#include "ldexp.h"
#include "ldlang.h"
#include "ldfile.h"

#include "ldemul.h"

#include "libbfd.h"

#include "coff/pic30.h"
#include "coff/internal.h"
#include "libcoff.h"

#include "../bfd/coff-pic30.h"

/* declared in bfd/coff-pic30.c */
extern bfd *handle_bfd;
extern unsigned char *handle_data;
extern struct handle_hash_table *handles;

extern boolean pic30_debug;
extern boolean pic30_data_init;
extern boolean pic30_has_data_init_option;
extern boolean pic30_handles;
extern boolean pic30_has_handles_option;
extern boolean pic30_pack_data;
extern boolean pic30_has_pack_data_option;
extern boolean pic30_has_stack_option;
extern unsigned int pic30_stack_size;
extern boolean pic30_has_heap_option;
extern boolean pic30_heap_required;
extern unsigned int pic30_heap_size;
extern boolean pic30_has_user_startup;
extern boolean pic30_smart_io;
extern boolean pic30_has_smart_io_option;
extern boolean pic30_has_user_default_interrupt;

extern bfd *init_bfd;
extern unsigned char *init_data;

extern bfd *isr_bfd;
extern unsigned char *isr_data;

struct traverse_hash_info
{
  bfd *abfd;
  asection *sec;
};

struct lma_adjust_info
{
  bfd_vma base;
  bfd_vma adjust;
  bfd_vma limit;
};

extern void pic30_rtype2howto
 PARAMS ((arelent *, struct internal_reloc *));

static struct bfd_hash_entry *handle_hash_newfunc
  PARAMS ((struct bfd_hash_entry *, struct bfd_hash_table *, const char *));

static void handle_hash_traverse
  PARAMS ((struct handle_hash_table *,
           boolean (*func) PARAMS ((struct bfd_hash_entry *, PTR)),
           PTR));

static struct handle_hash_table * handle_hash_init PARAMS ((void));

static arelent ** bfd_pic30_load_relocs PARAMS ((bfd *, asection *));

static void bfd_pic30_add_handle_sym PARAMS ((const char *));

static asection * bfd_pic30_create_section
  PARAMS ((bfd *, const char *, int, int));

static void bfd_pic30_create_symbol
  PARAMS ((bfd *, char *, asection *, int, int));

static void bfd_pic30_create_reloc
  PARAMS ((bfd *, int, int, int));

static void bfd_pic30_save_relocs PARAMS ((asection *));

static boolean bfd_pic30_create_jump_table_entry
  PARAMS ((struct bfd_hash_entry *, PTR));

static bfd * bfd_pic30_create_jump_table_bfd
  PARAMS ((bfd *));

static void bfd_pic30_add_bfd_to_link
  PARAMS ((bfd *, const char *));

static void bfd_pic30_redefine_global_symbol
  PARAMS ((const char *const , asection *, bfd_vma));

static void bfd_pic30_remove_archive_module
  PARAMS ((const char *const));

static void bfd_pic30_before_parse PARAMS ((void));

static boolean bfd_pic30_process_bfd_after_open
  PARAMS ((bfd *, struct bfd_link_info *));

static void gld${EMULATION_NAME}_before_parse PARAMS ((void));
static void gld${EMULATION_NAME}_before_allocation PARAMS ((void));
static void gld${EMULATION_NAME}_after_open PARAMS ((void));
static void gld${EMULATION_NAME}_finish PARAMS ((void));
static char *gld${EMULATION_NAME}_get_script PARAMS ((int *isfile));
static int  gld${EMULATION_NAME}_parse_args PARAMS((int, char **)); 
static void gld${EMULATION_NAME}_list_options PARAMS ((FILE *)); 

/*****************************************************************************/

/*
** Emulation Data Structures
*/

/* Emulation-specific Options */
const char * shortopts = "-D"; /* note: leading "-" is important */

/* Codes used for the long options with no short synonyms.  1000 isn't
   special; it's just an arbitrary non-ASCII char value that is
   different from the range used in lexsup.c  */
enum pic30_options {
  NO_HANDLES_OPTION = 1000,
  NO_DATA_INIT_OPTION,
  HEAP_OPTION,
  PACK_DATA_OPTION,
  STACK_OPTION,
  DATA_INIT_OPTION,
  NO_PACK_DATA_OPTION,
  HANDLES_OPTION,
  SMART_IO_OPTION,
  NO_SMART_IO_OPTION,
};

static struct option longopts[] = 
{ 
  { "data-init", no_argument, NULL, DATA_INIT_OPTION }, 
  { "debug", no_argument, NULL, 'D' }, 
  { "handles", no_argument, NULL, HANDLES_OPTION }, 
  { "heap", required_argument, NULL, HEAP_OPTION }, 
  { "no-data-init", no_argument, NULL, NO_DATA_INIT_OPTION },
  { "no-handles", no_argument, NULL, NO_HANDLES_OPTION }, 
  { "no-pack-data", no_argument, NULL, NO_PACK_DATA_OPTION }, 
  { "pack-data", no_argument, NULL, PACK_DATA_OPTION }, 
  { "stack", required_argument, NULL, STACK_OPTION }, 
  { "smart-io", no_argument, NULL, SMART_IO_OPTION },
  { "no-smart-io", no_argument, NULL, NO_SMART_IO_OPTION },
  { NULL,        no_argument,       NULL, 0                } 
}; 

/* Initial Size of the Handle Hash Table */
#define HANDLE_INIT 20

/* Symbol Tables */
static asymbol **syms;

static asymbol **symtab;
static int symptr;

/* Reloc Tables */
static arelent **rels;
static long relcount = 0;

static arelent *reltab =0;
static int relcnt = 0, relsize = 0;

/* The Number of Handle Relocations */
static unsigned long handle_relocs;

/* Dynamic Memory Info */
static unsigned int stack_base, stack_limit;
static unsigned int heap_base, heap_limit;
static unsigned int mem_limit;
static boolean has_const_section = false;
static boolean user_defined_stack = false;
#define PSV_BASE 0x8000
#define EEDATA_BASE 0x7ff000

/* Memory Reporting Info */
static bfd_size_type actual_prog_memory_used = 0;
static bfd_size_type data_memory_used = 0;

/* Section Lists */
struct pic30_section
{
  struct pic30_section *next;
  asection *sec;
};
static struct pic30_section *pic30_section_list;
static struct pic30_section *readonly_sections, *code_sections;

/* Free Memory List */
struct pic30_memory
{
  struct pic30_memory *next;
  bfd_vma addr;
  bfd_vma size;
  bfd_vma offset;
};
static struct pic30_memory *free_blocks;

/*
** smart_io Data Structure
**
** The following data structure is used to identify
** I/O function pairs that have redundant functionality.
** The "full_set" function is a standard I/O function
** such as printf(). The "reduced_set" function has the
** same type and parameter list but supports a reduced
** feature set, such as the integer-only _iprintf().
**
** If both members of a function pair are defined,
** the reduced_set function is considered redundant.
** The --smart-io option causes the linker to merge
** the two functions in order to conserve memory.
**
** Function merging is performed by the _after_open()
** emulation entry point, after all the input BFDs
** have been opened, but before memory is allocated.
**
** Function merging consists of several steps:
**
** 1. Lookup both function names in the Global Linker
**    Hash Table and load pointers to the corresponding
**    hash table entries.
**
** 2. Copy the (value) and (section) fields from the
**    full_set hash table entry to the reduced_set hash
**    table entry. This causes the reduced_set function
**    name to be a synonym for the full_set function name.
**
**    This operation is valid because:
**
**    (section) is a pointer into a BFD data structure.
**    When a BFD is opened, it remains locked in memory
**    until it is closed at the completion of the link.
**    Pointers into a BFD data structure are guaranteed
**    not to move.
**
**    (value) is an offset into an input section. Offsets
**    within an input section are determined by the
**    assembler and do not change during the link.
**
** 3. Loop through each input statement in the linker script
**    and find the BFD associated with the reduced_set
**    function, based on its module_name.
**
** 4. Traverse the Global Linker Hash Table and change the
**    type of any symbols that are defined and owned by
**    the reduced_set BFD to "undefined".
**
** 5. For each section attached to the reduced_set BFD,
**    set the SEC_NEVER_LOAD section flag and set the
**    section size to zero. This causes the reduced_set
**    function to be omitted from the final link.
*/

struct function_pair_type
{
  char *full_set;
  char *reduced_set;
  char *module_name;
};

const struct function_pair_type function_pairs[] =
  {
    /* iprintf() */
    { "_printf",    "__iprintf",   "printf.io"    },
    { "__Printf",   "___iPrintf",  "xprintf.io"   },
    { "__Putfld",   "___iPutfld",  "xputfld.io"   },

    { "__dprintf",  "__iprintf",   "printf.io"    },
    { "___dPrintf", "___iPrintf",  "xprintf.io"   },
    { "___dPutfld", "___iPutfld",  "xputfld.io"   },

    /* ifprintf() */
    { "_fprintf",   "__ifprintf",  "fprintf.io"   },
    { "__dfprintf", "__ifprintf",  "fprintf.io"   },

    /* isprintf() */
    { "_sprintf",   "__isprintf",  "sprintf.io"   },
    { "__dsprintf", "__isprintf",  "sprintf.io"   },

    /* ivprintf() */
    { "_vprintf",   "__ivprintf",  "vprintf.io"   },
    { "__dvprintf", "__ivprintf",  "vprintf.io"   },

    /* ivfprintf() */
    { "_vfprintf",  "__ivfprintf",  "vfprintf.io" },
    { "__dvfprintf","__ivfprintf",  "vfprintf.io" },

    /* ivsprintf() */
    { "_vsprintf",  "__ivsprintf",  "vsprintf.io" },
    { "__dvsprintf","__ivsprintf",  "vsprintf.io" },

    /* isnprintf() */
    { "_snprintf",  "__isnprintf",  "snprintf.io" },
    { "__dsnprintf","__isnprintf",  "snprintf.io" },

    /* ivsnprintf() */
    { "_vsnprintf", "__ivsnprintf", "vsnprint.io" },
    { "__dvsnprintf","__ivsnprintf","vsnprint.io" },

    /* iscanf() */
    { "_scanf",     "__iscanf",     "scanf.io"    },
    { "__Scanf",    "___iSscanf",   "xscanf.io"   },
    { "__Getfld",   "___iGetfld",   "xgetfld.io"  },

    { "__dscanf",   "__iscanf",     "scanf.io"    },
    { "___dScanf",  "___iSscanf",   "xscanf.io"   },
    { "___dGetfld", "___iGetfld",   "xgetfld.io"  },

    /* ifscanf() */
    { "_fscanf",    "__ifscanf",    "fscanf.io"   },
    { "__dfscanf",  "__ifscanf",    "fscanf.io"   },

    /* isscanf() */
    { "_sscanf",    "__isscanf",    "sscanf.io"   },
    { "__dsscanf",  "__isscanf",    "sscanf.io"   },

    /* null terminator */
    { 0, 0, 0 },
  };

/*****************************************************************************/

/*
** BFD Utility Routines
*/

/* print reloc - internal form (arelent) */
#define DUMP_RELOC(n,r)                              \
{                                                    \
   fprintf (stderr,"  %s sym %d, addr %d, addend %d\n", \
	   n, (*(r->sym_ptr_ptr))->name,             \
	   r->address, r->addend);                   \
}

/* print reloc - canonical form (arelent) */
#define DUMP_CANON_RELOC(n,r)                              \
{                                                    \
   fprintf (stdout,"  %s sym %s, addr %ld, addend %ld\n", \
	   n, (*(r->sym_ptr_ptr))->name,             \
	   r->address, r->addend);                   \
}

/* print reloc - external form */
#define DUMP_RELOC2(n,r)                     \
{                                            \
   fprintf (stderr,"  %s sym %d, r_vaddr 0x%lx\n", \
	   n, (int) r->r_symndx, (long) r->r_vaddr);      \
}

/*****************************************************************************/

/*
** Handle Hash Table Routines
**
** We use a hash table to keep track of
** unique symbols referenced in handle relocations.
**
** Each symbol has an entry in the jump table
** which is resolved at reloc time.
**
** The total size of the jump table is determined
** by the number of unique symbols.
**
** This hash table is derived from the basic
** table provided in /bfd/hash.c
*/


/*
** Create a new hash table entry
*/
static struct bfd_hash_entry *
handle_hash_newfunc (entry, table, string)
     struct bfd_hash_entry *entry;
     struct bfd_hash_table *table;
     const char *string;
{
  struct handle_hash_entry *ret = (struct handle_hash_entry *) entry;

  /* Allocate the structure if it has not already been allocated by a
     subclass.  */
  if (ret == (struct handle_hash_entry *) NULL)
    ret = ((struct handle_hash_entry *)
	   bfd_hash_allocate (table, sizeof (struct handle_hash_entry)));
  if (ret == (struct handle_hash_entry *) NULL)
    return NULL;

  /* Call the allocation method of the superclass.  */
  ret = ((struct handle_hash_entry *)
	 bfd_hash_newfunc ((struct bfd_hash_entry *) ret, table, string));

  if (ret)
    {
      /* Initialize the local fields.  */
      ret->index    = -1;
      ret->sym      = 0;
    }

  return (struct bfd_hash_entry *) ret;
}

#if 0
/*
** Look up a hash table entry
*/
#define handle_hash_lookup(t, string, create, copy) \
  ((struct handle_hash_entry *) \
   bfd_hash_lookup (&(t)->table, (string), (create), (copy)))
#endif


/*
** Traverse the hash table
**
** Calls a function for each entry
*/
static void
handle_hash_traverse (table, func, info)
     struct handle_hash_table *table;
     boolean (*func) PARAMS ((struct bfd_hash_entry *, PTR));
     PTR info;
{
  bfd_hash_traverse (&table->table,
		     ((boolean (*) PARAMS ((struct bfd_hash_entry *, PTR)))
		      func),
		     info);
}


/*
** Create a new hash table
*/
static struct handle_hash_table *
handle_hash_init ()
{
  struct handle_hash_table *table;

  table = ((struct handle_hash_table *)
	   bfd_alloc (output_bfd, sizeof (struct handle_hash_table)));
  if (table == NULL)
    return NULL;

  if (! bfd_hash_table_init_n (&table->table, handle_hash_newfunc, HANDLE_INIT))
    {
      free (table);
      return NULL;
    }

  table->num = 0;

  return table;
}

#if 0
/*
** Free a hash table
*/
static void
handle_hash_free (table)
     struct handle_hash_table *table;
{
  bfd_hash_table_free (&table->table);
  free (table);
}
#endif

/*****************************************************************************
**
** Section List Routines
**
** Maintain a list of sections in order
** of ascending load address.
**
** Used for creating memory usage reports.
**
*/

/*
** Create a new list
**
** - first item is null
*/
static void
pic30_init_section_list(lst)
     struct pic30_section **lst;
{
  *lst = ((struct pic30_section *)
         xmalloc(sizeof(struct pic30_section)));
  (*lst)->next = 0;
  (*lst)->sec = 0;
}


/*
** Add a section to the list
**
** - in order of increasing load address
** - this function is called via bfd_map_over_sections()
*/
static void
pic30_build_section_list (abfd, sect, fp)
     bfd *abfd ATTRIBUTE_UNUSED ;
     asection *sect;
     PTR fp ATTRIBUTE_UNUSED ;
{
  struct pic30_section *new, *s, *prev;
  int done = 0;

  /* create a new element */
  new = ((struct pic30_section *) 
         xmalloc(sizeof(struct pic30_section)));
  new->sec  = sect;

  /* insert it at the right spot */
  prev = pic30_section_list;
  for (s = prev; s != NULL; s = s->next)
    {
      if (s->sec && (new->sec->lma < s->sec->lma))
        {
          prev->next = new;
          new->next = s;
          done++;
          break;
        }
      prev = s;
    }

  if (!done)
    {
      prev->next = new;
      new->next = NULL;
    }
} /* static void pic30_build_section_list (...) */


/*
** Free a section list
*/
static void
pic30_free_section_list(lst)
     struct pic30_section **lst;
{
  struct pic30_section *s, *next;

  if (!(*lst))
    return;

  for (s = *lst; s != NULL; s = next)
    {
      next = s->next;
      free(s);
    }

  *lst = NULL;
} /* static void pic30_free_section_list (...) */


/*
** Append two section lists
*/
static void
pic30_append_section_lists(lst1, lst2)
     struct pic30_section *lst1, *lst2;
{
  struct pic30_section *s, *tail;

  if (!(lst1))
    return;

  for (s = lst1; s != NULL; s = s->next)
    tail = s;

  tail->next = lst2;

} /* static void pic30_free_section_list (...) */

/*****************************************************************************/

/*****************************************************************************
**
** Memory Allocation Routines
**
** Used by bfd_pic30_adjust_psv_sections()
** to re-allocate sections above a certain
** threshold.
**
*/


/*
** Build a list of READONLY sections
**  in order of decreasing size
**
** - this function is called via bfd_map_over_sections()
*/
static void
pic30_build_readonly_section_list (abfd, sect, p)
     bfd *abfd ATTRIBUTE_UNUSED ;
     asection *sect;
     PTR p;
{
  struct lma_adjust_info *info = (struct lma_adjust_info *) p;
  struct pic30_section *new, *s, *prev;
  int done = 0;

  /* exit if not READONLY, or below threshold, or above limit */
  if (((sect->flags & SEC_READONLY) == 0)
      || (sect->lma < info->base) || (sect->lma > info->limit))
    return;

  /* create a new element */
  new = ((struct pic30_section *) 
         xmalloc(sizeof(struct pic30_section)));
  new->sec  = sect;

  /* insert it at the right spot */
  prev = readonly_sections;
  for (s = prev; s != NULL; s = s->next)
    {
      if (s->sec && (new->sec->_raw_size > s->sec->_raw_size))
        {
          prev->next = new;
          new->next = s;
          done++;
          break;
        }
      prev = s;
    }

  if (!done)
    {
      prev->next = new;
      new->next = NULL;
    }
} /* static void pic30_build_readonly_section_list (...) */


/*
** Build a list of CODE sections
**  in order of decreasing size
**
** - this function is called via bfd_map_over_sections()
*/
static void
pic30_build_code_section_list (abfd, sect, p)
     bfd *abfd ATTRIBUTE_UNUSED ;
     asection *sect;
     PTR p;
{
  struct lma_adjust_info *info = (struct lma_adjust_info *) p;
  struct pic30_section *new, *s, *prev;
  int done = 0;

  /* exit if not CODE, or below threshold, or above limit */
  if (((sect->flags & SEC_CODE) == 0)
      || (sect->lma < info->base) || (sect->lma > info->limit))
    return;

  /* create a new element */
  new = ((struct pic30_section *) 
         xmalloc(sizeof(struct pic30_section)));
  new->sec  = sect;

  /* insert it at the right spot */
  prev = code_sections;
  for (s = prev; s != NULL; s = s->next)
    {
      if (s->sec && (new->sec->_raw_size > s->sec->_raw_size))
        {
          prev->next = new;
          new->next = s;
          done++;
          break;
        }
      prev = s;
    }

  if (!done)
    {
      prev->next = new;
      new->next = NULL;
    }
} /* static void pic30_build_code_section_list (...) */


/*
** Create a list of free memory blocks
**
** - first item is null
*/
static void
pic30_init_memory_list(lst)
     struct pic30_memory **lst;
{
  *lst = ((struct pic30_memory *)
         xmalloc(sizeof(struct pic30_memory)));
  (*lst)->next = 0;
  (*lst)->addr = 0;
  (*lst)->size = 0;
  (*lst)->offset = 0;
}


/*
** Add a node to a memory list
**  in order of increasing size
*/
static void
pic30_add_to_memory_list (lst, addr, size)
     struct pic30_memory *lst;
     bfd_vma addr, size;
{
  struct pic30_memory *new, *b, *prev;
  int done = 0;

  /* create a new element */
  new = ((struct pic30_memory *) 
         xmalloc(sizeof(struct pic30_memory)));
  new->addr  = addr;
  new->size  = size;
  new->offset = 0;

  /* insert it at the right spot */
  prev = lst;
  for (b = prev; b != NULL; b = b->next)
    {
      if (new->size < b->size)
        {
          prev->next = new;
          new->next = b;
          done++;
          break;
        }
      prev = b;
    }

  if (!done)
    {
      prev->next = new;
      new->next = NULL;
    }
} /* static void pic30_add_to_memory_list (..) */


/*
** Find a suitable free block - static
**
** This function scans the free memory list
** looking for a buffer that will accomodate
** a static section (fixed address).
**
** If a suitable block is found, fill in the
** offset field to reflect the section starting
** address and return the block.
**
** If a suitable block can not be found, return NULL.
*/
static struct pic30_memory *
pic30_static_assign_memory (lst, size, address)
     struct pic30_memory *lst;
     bfd_vma size, address;
{
  struct pic30_memory *b, *prev;
  bfd_vma offset;

  prev = lst;
  for (b = prev; b != NULL; b = b->next)
    {
      if ((b->size < size) || (b->addr > address))
        continue;

      offset = address - b->addr;
      if (b->size >= (offset + size))
        {
          b->offset = offset;
          return b;
        }
    }

  /*
  ** if we get here, a suitable block could not be found
  */
  return (struct pic30_memory *) NULL;
} /* static struct pic30_memory * pic30_static_assign_memory (...) */


/*
** Find a suitable free block
**
** This function scans the free memory list
** looking for the best fit based on section
** size, alignment power, and READONLY status.
**
** If a suitable block is found, fill in the
** offset field to reflect the section starting
** address and return the block.
**
** If a suitable block can not be found, return NULL.
*/
static struct pic30_memory *
pic30_best_fit_memory (lst, size, align_power, readonly)
     struct pic30_memory *lst;
     bfd_vma size;
     unsigned int align_power;
     boolean readonly;
{
  struct pic30_memory *b;
  bfd_vma mask = ~((~0) << align_power);
  bfd_vma new_address, new_size, next_page;
  bfd_vma align_offset, page_offset;

  for (b = lst; b != NULL; b = b->next)
    {
      if (b->size < size)
        continue;

      /* compute alignment offset */
      new_address = (b->addr + mask) & (~mask);
      align_offset = new_address - b->addr;

#if 0
      printf("\nalign_power = %x\n", align_power);
      printf("mask = %lx\n", mask);
      printf("block addr = %lx\n", b->addr);
      printf("new_address = %lx\n", new_address);
      printf("align_offset = %lx\n", align_offset);
#endif

      /* compute page blocking offset */
      page_offset = 0;
      if (readonly)
        {
          next_page = ((b->addr + align_offset) & ~ 0x7FFF) + 0x8000;
          if ((b->addr + align_offset + size) > next_page)
            {
              /*
              ** This block will cross a PSV page boundary, so we
              ** must advance to the next page. Page boundaries (2^15)
              ** will satisfy any section alignment requirement. 
              */
              page_offset = next_page - (b->addr + align_offset);
#if 0
              printf("next_page = %lx\n", next_page);
              printf("page_offset = %lx\n", page_offset);
#endif
            }
        }

      /* see if this block meets all requirements */
      new_size = size + align_offset + page_offset;
      if (b->size >= new_size)
        {
          /* if so, fill in offset and return it */
          b->offset = align_offset + page_offset;
          return b;
        }
    } /* for (b = prev; b != NULL; b = b->next) */

  /*
  ** if we get here, a suitable block could not be found
  */
  return (struct pic30_memory *) NULL;
} /* static struct pic30_memory * pic30_best_fit_memory (...) */


/*
** Remove a block from a memory list
*/
static void
pic30_remove_from_memory_list (lst, block)
     struct pic30_memory *lst;
     struct pic30_memory *block; 
{
  struct pic30_memory *b, *prev, *next;

  prev = lst;
  for (b = prev; b != NULL; b = next)
    {
      next = b->next;
      if (b == block)
        {
          prev->next = next;
          free(b);
          break;
        }
      prev = b;
    }

} /* static void pic30_remove_from_memory_list(..) */


/*
** Free a memory list
*/
static void
pic30_free_memory_list(lst)
     struct pic30_memory **lst;
{
  struct pic30_memory *s, *next;

  if (!(*lst))
    return;

  for (s = *lst; s != NULL; s = next)
    {
      next = s->next;
      free(s);
    }

  *lst = NULL;
} /* static void pic30_free_memory_list (...) */


/*
** Utility Routines
**
** These functions are called at various
** points in the emulation.
*/


/*
** Load in the relocs for a section
*/
static arelent **
bfd_pic30_load_relocs (abfd, sec)
     bfd *abfd;
     asection *sec;
{
  long relsize;
  arelent **relpp;

  relcount = 0;  /* default, no relocs in this section */

  if ((bfd_is_abs_section (sec)) ||
      (bfd_is_und_section (sec)) ||
      (bfd_is_com_section (sec)) ||
      (sec->flags & SEC_RELOC) == 0)
    return NULL;

  /* get the size needed for a reloc buffer */
  relsize = bfd_get_reloc_upper_bound (abfd, sec);
  if (relsize < 0)
    {
      fprintf( stderr, "Link Error: can't read relocs from %s\n",
               (char *) bfd_get_filename (abfd));
      abort();
    }

  if (relsize == 0)
    return NULL;
  else
    {
      /* get a buffer to hold the relocs */
      relpp = (arelent **) malloc (relsize);
      if (!relpp)
        {
          fprintf( stderr, "Link Error: not enough memory for relocs\n");
          abort();
        }

      /* load the symbol table */
      syms = _bfd_generic_link_get_symbols (abfd);

      /* canonicalize the relocs */
      relcount = bfd_canonicalize_reloc (abfd, sec, relpp, syms);

      /* check for errors */
      if (relcount < 0)
        {
          fprintf( stderr, "Link Error: can't canonicalize relocs from %s\n",
                   (char *) bfd_get_filename (abfd));
          abort();
        }
      else if (relcount == 0)
        {
          free (relpp);
          return NULL;
        }
      else
        return (relpp);
    }
} /* static arelent ** bfd_pic30_load_relocs (...)*/


/*
** Add a symbol to the handle hash table
*/
static void
bfd_pic30_add_handle_sym(sym)
     const char * sym;
{
  struct handle_hash_entry * handle;
  char *name;

  /* if no hash table exists, create one */
  if (!handles)
    handles = handle_hash_init();
    
  /* if we already have this entry... */
  handle = handle_hash_lookup (handles, sym, 0, 0);
  if (handle)
    return; /* simply return */

  /* else make a permanent copy of the symbol name */
  name = (char *) bfd_alloc (output_bfd, (strlen (sym) + 1));
  if (!name)
    {
      fprintf( stderr, "Link Error: not enough memory for handle sym\n");
      abort();
    }
  strcpy( name, sym);

  if (pic30_debug)
    printf("  Adding handle %s  %x\n", name, handles->num);

  /* create a new handle for this symbol */
  handle = handle_hash_lookup (handles, name, 1, 0);

  /* initialize it */
  handle->index = handles->num++;
  handle->sym   = name;

} /* static void bfd_pic30_add_handle_sym (...)*/


/*
** Create a new section
*/
static asection *
bfd_pic30_create_section (abfd, name, flags, align)
     bfd *abfd;
     const char *name;
     int flags;
     int align;
{
  asection *sec;
  asymbol *sym;

  if (pic30_debug) 
    printf("\nCreating input section \"%s\"\n", name);

  sec = bfd_make_section_old_way (abfd, name);
  bfd_set_section_flags (abfd, sec, flags | SEC_ALLOC | SEC_LOAD | SEC_KEEP);
  bfd_set_section_alignment (abfd, sec, align);
  sec->output_section = sec;

  /* add a symbol for the section name */
  sym = bfd_make_empty_symbol (abfd);
  symtab[symptr++] = sym;
  sym->name = sec->name;
  sym->section = sec;
  sym->flags = BSF_LOCAL;
  sym->value = 0;

  return sec;
} /* static asection * bfd_pic30_create_section (...)*/


/*
** Create a new symbol
*/
static void
bfd_pic30_create_symbol (abfd, name, sec, flags, addr)
     bfd *abfd;
     char *name;
     asection *sec;
     int flags;
     int addr;
{
  asymbol *sym;

/*   if (pic30_debug)  */
/*     printf("  Adding symbol %s\n", name); */

  sym = bfd_make_empty_symbol (abfd);
  sym->name = name;
  sym->section = sec;
  sym->flags = flags;
  sym->value = addr;
  symtab[symptr++] = sym;
} /* static void bfd_pic30_create_symbol (...)*/


/*
** Create a new relocation
*/
static void
bfd_pic30_create_reloc (abfd, address, which_howto, symidx)
     bfd *abfd;
     int address;
     int which_howto;
     int symidx;
{
  if (relcnt >= (relsize - 1))
    {
      relsize += 10;
      if (reltab)
        {
          if (pic30_debug)
            printf("  Resizing the reloc table %x\n", relsize * sizeof (arelent));
          reltab = (arelent *) bfd_realloc (reltab, relsize * sizeof (arelent));
        }
      else
	reltab = (arelent *) bfd_malloc (relsize * sizeof (arelent));
    }
  reltab[relcnt].address = address;
  reltab[relcnt].addend = 0;
  reltab[relcnt].howto = bfd_reloc_type_lookup (abfd, which_howto);
  reltab[relcnt].sym_ptr_ptr = symtab + symidx;
/*   if (pic30_debug)  */
/*     printf("  Added reloc for %s\n", (*(reltab[relcnt].sym_ptr_ptr))->name); */
  relcnt++;
} /* static void bfd_pic30_create_reloc (...)*/


/*
** Save the reloc table into a section
*/
static void
bfd_pic30_save_relocs (asection *sec)
{
  int i;
  sec->relocation = reltab;
  sec->reloc_count = relcnt;
  sec->orelocation = (arelent **) bfd_alloc (output_bfd,
                                             (relcnt + 1) * sizeof (arelent *));
  for (i = 0; i < relcnt; i++)
    sec->orelocation[i] = sec->relocation + i;
  sec->orelocation[relcnt] = 0;
  sec->flags |= SEC_RELOC;
  reltab = 0;
  relcnt = relsize = 0;
} /* static void bfd_pic30_save_relocs (...)*/


/*
** Print a handle hash table entry 
**
** This routine is called by handle_hash_traverse()
*/
static boolean
bfd_pic30_dump_handle_entry (h, p)
     struct bfd_hash_entry *h;
     PTR p ATTRIBUTE_UNUSED;
{
  struct handle_hash_entry *handle = (struct handle_hash_entry *) h;

  printf("  %x\t%s\n", handle->index, handle->sym);

  /* traverse the entire hash table */
  return true;
} /* static boolean bfd_pic30_dump_handle_entry (...)*/


/*
** Create a new jump table entry 
**
** This routine is called by handle_hash_traverse()
**
** Note: unlike other offsets and lengths, relocation
**       addresses are specified in bytes
*/
static boolean
bfd_pic30_create_jump_table_entry (h, p)
     struct bfd_hash_entry *h;
     PTR p;
{
  struct handle_hash_entry *handle = (struct handle_hash_entry *) h;
  struct traverse_hash_info *info = (struct traverse_hash_info *) p;

  /* create a symbol for the jump target */
  bfd_pic30_create_symbol (info->abfd,
                           handle->sym,
                           /* info->sec, BSF_GLOBAL, 0); */
                           bfd_und_section_ptr, BSF_GLOBAL, 0);

  /* create two relocs for the jump target */
  bfd_pic30_create_reloc (info->abfd,
                          handle->index * 8,
                          BFD_RELOC_PIC30_PGM_ADDR_LSB, symptr - 1);
  bfd_pic30_create_reloc (info->abfd,
                          (handle->index * 8) + 4,
                          BFD_RELOC_PIC30_PGM_ADDR_MSB, symptr - 1);

  /* traverse the entire hash table */
  return true;
} /* static boolean bfd_pic30_create_jump_table_entry (...)*/


/*
** Create a bfd for the handle jump table
*/
static bfd *
bfd_pic30_create_jump_table_bfd (parent)
     bfd *parent;
{
  bfd_size_type size, i;
  bfd *abfd;
  asection *sec;
  char *oname;
  unsigned char *d;

  struct traverse_hash_info info;
  struct traverse_hash_info *tinfo = &info;

  if (pic30_debug)
    {
      /* print out the handle hash table entries */
      printf("\nHash Table Entries:\n");
      handle_hash_traverse (handles, bfd_pic30_dump_handle_entry, (PTR) 0);
    }

  /* create a bare-bones bfd */
  oname = (char *) bfd_alloc (output_bfd, 20);
  sprintf (oname, "jump_table");
  abfd = bfd_create (oname, parent);
  bfd_find_target ("coff-pic30", abfd);
  bfd_make_writable (abfd);

  bfd_set_format (abfd, bfd_object);
  bfd_set_arch_mach (abfd, bfd_arch_pic30, 0);

  /* create a symbol table (room for num+1 entries) */
  symptr = 0;
  symtab = (asymbol **) bfd_alloc (output_bfd,
                                   (handles->num + 1) * sizeof (asymbol *));

  /* create a bare-bones section */
  sec = bfd_pic30_create_section (abfd, ".handle",
                                  SEC_HAS_CONTENTS | SEC_IN_MEMORY, 1);
  size = (bfd_size_type) handles->num * 8;
  bfd_set_section_size (abfd, sec, size);

  /* allocate memory for the jump table */
  handle_data = (unsigned char *) bfd_alloc (output_bfd, size);
  if (!handle_data)
    {
      fprintf( stderr, "Link Error: not enough memory for jump table\n");
      abort();
    }
  sec->contents = handle_data;

  /* fill it with GOTO opcodes */
  d = handle_data;
  for (i=1; i<size; i+=8)
    {
      *d++ = 0; *d++ = 0;
      *d++ = 0x04; *d++ = 0;
      *d++ = 0; *d++ = 0;
      *d++ = 0; *d++ = 0;
    }

  /* create symbols and relocs for all the handles */
  tinfo->abfd = abfd;
  tinfo->sec = sec;
  handle_hash_traverse (handles, bfd_pic30_create_jump_table_entry, tinfo);

  /* put in the relocs */
  bfd_pic30_save_relocs (sec);

  /* put in the symbol table */
  bfd_set_symtab (abfd, symtab, symptr);

  /* and the new section */
  bfd_set_section_contents (abfd, sec, handle_data, 0, size);

  /* finish it */
  if (!bfd_make_readable (abfd))
      {
        fprintf( stderr, "Link Error: can't make jump table readable\n");
        abort();
      }


  if (pic30_debug)
    /* DEBUG -- load in the symbols and print them out */
    {
      long size, num,i;
      asymbol **symbols;

      printf("\nSymbols in bfd \"%s\":\n", abfd->filename);
      size = bfd_get_symtab_upper_bound (abfd);
      if (size < 0)
        {
          fprintf( stderr, "Link Error: error accessing symbol table\n");
          abort();
        }

      symbols = (asymbol **) bfd_alloc(output_bfd, size);
      if (!symbols)
        {
          fprintf( stderr, "Link Error: not enough memory for symbol table\n");
          abort();
        }

      num = bfd_canonicalize_symtab (abfd, symbols);
      if (num < 0)
        {
          fprintf( stderr, "Link Error: error processing symbol table\n");
          free (symbols);
          abort();
        }
      else
        {
          for (i=0; i<num; i++)
            printf("  %s\n", bfd_asymbol_name(symbols[i]));
        }
    }

  if (pic30_debug)
    /* DEBUG -- load in the relocs and print them out */
    {
      arelent **p;

      printf("\nRelocs in bfd \"%s\", section \"%s\":\n",
             abfd->filename, sec->name);
      rels = bfd_pic30_load_relocs(abfd, sec);
      for (p = rels; relcount && *p != (arelent *) NULL; p++, relcount--)
        {
          arelent *q = *p;
          DUMP_CANON_RELOC(q->howto->name, q);
        }
      free (rels);
    }


  return abfd;
} /* static bfd * bfd_pic30_create_jump_table_bfd (...)*/


/*
** Create a bfd for the data init template
*/
static bfd *
bfd_pic30_create_data_init_bfd (parent)
     bfd *parent ATTRIBUTE_UNUSED;
{
  bfd_size_type size;
  bfd *abfd;
  asection *sec;
  char *oname;

  /* create a bare-bones bfd */
  oname = (char *) bfd_alloc (output_bfd, 20);
  sprintf (oname, "data_init");
  abfd = bfd_create (oname, parent);
  bfd_find_target ("coff-pic30", abfd);
  bfd_make_writable (abfd);

  bfd_set_format (abfd, bfd_object);
  bfd_set_arch_mach (abfd, bfd_arch_pic30, 0);

  /* create a symbol table (room for 1 entry) */
  symptr = 0;
  symtab = (asymbol **) bfd_alloc (output_bfd, sizeof (asymbol *));

  /* create a bare-bones section */
  sec = bfd_pic30_create_section (abfd, ".dinit", SEC_HAS_CONTENTS, 1);
  size = 0; /* will update later */
  bfd_set_section_size (abfd, sec, size);

  /* put in the symbol table */
  bfd_set_symtab (abfd, symtab, symptr);

  /* will add section contents later */

  /* finish it */
  if (!bfd_make_readable (abfd))
    {
      fprintf( stderr, "Link Error: can't make data init readable\n");
      abort();
    }

  return abfd;
} /* static bfd * bfd_pic30_create_data_init_bfd (...)*/


/*
** Create a bfd for the default interrupt handler
*/
static bfd *
bfd_pic30_create_default_isr_bfd  (parent)
     bfd *parent;
{
  bfd_size_type size;
  bfd *abfd;
  asection *sec;
  char *oname;
  unsigned char *d;

  /* create a bare-bones bfd */
  oname = (char *) bfd_alloc (output_bfd, 20);
  sprintf (oname, "default_isr");
  abfd = bfd_create (oname, parent);
  bfd_find_target ("coff-pic30", abfd);
  bfd_make_writable (abfd);

  bfd_set_format (abfd, bfd_object);
  bfd_set_arch_mach (abfd, bfd_arch_pic30, 0);

  /* create a symbol table (room for 2 entries) */
  symptr = 0;
  symtab = (asymbol **) bfd_alloc (output_bfd, 2 * sizeof (asymbol *));

  /* create a bare-bones section */
  sec = bfd_pic30_create_section (abfd, ".isr",
                                  SEC_HAS_CONTENTS | SEC_IN_MEMORY, 1);
  size = (bfd_size_type) 4;
  bfd_set_section_size (abfd, sec, size);

  /* allocate memory */
  isr_data = (unsigned char *) bfd_alloc (output_bfd, size);
  if (!isr_data)
    {
      fprintf( stderr, "Link Error: not enough memory for default isr\n");
      abort();
    }
  sec->contents = isr_data;

  /* fill it with RESET opcode */
  d = isr_data;
  *d++ = 0; *d++ = 0;
  *d++ = 0xFE; *d++ = 0;

  /* create global label at offset zero */
  bfd_pic30_create_symbol (abfd, "__DefaultInterrupt", sec, BSF_GLOBAL, 0);

  /* put in the symbol table */
  bfd_set_symtab (abfd, symtab, symptr);

  /* and the new section */
  bfd_set_section_contents (abfd, sec, isr_data, 0, size);

  /* finish it */
  if (!bfd_make_readable (abfd))
      {
        fprintf( stderr, "Link Error: can't make default isr readable\n");
        abort();
      }

  return abfd;
} /* static bfd * bfd_pic30_create_default_isr_bfd (...)*/


/*
** Add a new bfd to the link
*/
static void
bfd_pic30_add_bfd_to_link (abfd, name)
     bfd *abfd;
     const char *name;
{
  lang_input_statement_type *fake_file;
  fake_file = lang_add_input_file (name,
				   lang_input_file_is_fake_enum,
				   NULL);
  fake_file->the_bfd = abfd;
  ldlang_add_file (fake_file);
  if (! bfd_link_add_symbols (abfd, &link_info))
    einfo (_("%F%B: could not add symbols: %E\n"), abfd);

  return;
} /* static void bfd_pic30_add_bfd_to_link (...)*/


/*
** This routine is called via bfd_map_over_sections().
**
** Scan for loadable DATA sections
** and accumulate the total size
*/
static void
bfd_pic30_scan_data_sections (abfd, sect, p)
     bfd *abfd ATTRIBUTE_UNUSED ;
     asection *sect;
     PTR p;
{
#define DATA_RECORD_HEADER_SIZE 12

  int *data_size = (int *) p;

  if (p == (int *) NULL)
    return;

  /* skip any persistent data section */
  if (strstr(sect->name, ".pbss") != 0)
    {
      /* issue a warning if DATA values are present */
      if ((sect->flags & SEC_DATA) && (sect->_raw_size > 0))
        einfo(_("%P: Warning: initial values were specified for a persistent"
                " data section (%s). These values will be ignored.\n"),
              sect->name);

      if (pic30_debug)
        printf("  %s (skipped), size = %x bytes\n",
               sect->name, (unsigned int) sect->_raw_size / 2);
      return;
    }

  /* process BSS-type sections */
  if (((sect->flags & (SEC_ALLOC | SEC_LOAD | SEC_DATA)) == SEC_ALLOC) &&
      (sect->_raw_size > 0))
    {
      *data_size += DATA_RECORD_HEADER_SIZE;
      if (pic30_debug)
        printf("  %s (bss), size = %x bytes, template += %x pwords\n",
               sect->name, (unsigned int) sect->_raw_size / 2,
               DATA_RECORD_HEADER_SIZE / 4);
    }

  /* process DATA-type sections */
  if (((sect->flags & SEC_DATA) == SEC_DATA) && (sect->_raw_size > 0))
    {
      int count = sect->_raw_size / 2; /* ignore phantom in data image */
      int delta = DATA_RECORD_HEADER_SIZE +
                  (pic30_pack_data ? (((count + 2) / 3) * 4) :
                                     (((count + 1) / 2) * 4));
      *data_size += delta;
      
      /* make section not LOADable */
      sect->flags &= ~ SEC_LOAD;
      sect->flags |= SEC_NEVER_LOAD;

      if (pic30_debug)
        printf("  %s (data), size = %x bytes, template += %x pwords\n",
               sect->name, (unsigned int) sect->_raw_size / 2, delta / 4);
    } /* process DATA-type sections */

  return;
} /*static void bfd_pic30_scan_data_sections (...)*/


/*
** This routine is called via bfd_map_over_sections()
** when the --no-data-init option is specified.
**
** Scan for loadable DATA sections...if found,
** mark them NEVER_LOAD and issue a warning.
*/
static void
bfd_pic30_skip_data_sections (abfd, sect, p)
     bfd *abfd ATTRIBUTE_UNUSED ;
     asection *sect;
     PTR p ATTRIBUTE_UNUSED ;
{
  /* process DATA-type sections */
  if (((sect->flags & SEC_DATA) == SEC_DATA) && (sect->_raw_size > 0))
    {      
      /* make section not LOADable */
      sect->flags &= ~ SEC_LOAD;
      sect->flags |= SEC_NEVER_LOAD;

      /* issue a warning */
      einfo(_("%P: Warning: data initialization has been turned off,"
              " therefore section %s will not be initialized.\n"), sect->name);

      if (pic30_debug)
        printf("  %s (skipped), size = %x\n",
               sect->name, (unsigned int) sect->_raw_size / 2);
    } /* process DATA-type sections */

  return;
} /*static void bfd_pic30_skip_data_sections (...)*/



/*
** Print diagnostic section info
*/
static void
bfd_pic30_print_section_header()
{
  printf("  %8s %5s %6s %6s %6s\n",
         " section", "flags", "   VMA", "   LMA", "length");
  printf("  %8s %4s %6s %6s %6s\n",
         " -------", "-----", "------", "------", "------");
}

static void
bfd_pic30_report_sections (abfd, sect, fp) 
     bfd *abfd;
     asection *sect;
     PTR fp ATTRIBUTE_UNUSED ;
{
  printf("  %8s %5x %6x %6x %6x %s\n", sect->name, sect->flags,
         (unsigned int) sect->vma,
         (unsigned int) sect->lma,
         (unsigned int) sect->_raw_size / bfd_octets_per_byte (abfd),
         sect->user_set_vma ? "(fixed)" : "");
  return;
}


/*
** Strip zero-length sections
*/
static void
pic30_strip_sections (abfd)
     bfd *abfd;
{
  asection *sec, *prev;
  
  if (pic30_debug)
    printf("\nLooking for zero-length sections:\n");

  sec = abfd->sections;
  if ((sec == NULL) || (sec->next == NULL))
    return;

  prev = sec;
  sec = sec->next; /* never strip the first section */

  /* loop through the sections in this bfd */
  for (; sec != NULL; sec = sec->next)
    {
      /* remove sections with size = 0 */
      if (sec->_raw_size == 0)
        {
        prev->next = sec->next;
        abfd->section_count -= 1;
        if (pic30_debug)
          printf("  Stripping section %s\n", sec->name);
        }
      else
        prev = sec;
    }
  return;
} /* static void pic30_strip_sections (...)*/


/*
** Return a pointer to bfd_link_hash_entry
** if a global symbol is defined;
** else return zero.
*/
struct bfd_link_hash_entry *
bfd_pic30_is_defined_global_symbol (name)
     const char *const name;
{
    struct bfd_link_hash_entry *h;
    h = bfd_link_hash_lookup (link_info.hash, name, false, false, true);

    if ((h != (struct bfd_link_hash_entry *) NULL) &&
        (h->type == bfd_link_hash_defined))
      return h;
    else
      return (struct bfd_link_hash_entry *) NULL;
}


/*
** Re-define a global symbol
*/
static void
bfd_pic30_redefine_global_symbol (name, sec, value)
     const char *const name;
     asection *sec;
     bfd_vma value;
{
  struct bfd_link_hash_entry *h;
  h = bfd_link_hash_lookup (link_info.hash, name, false, false, true);
  if ((h != (struct bfd_link_hash_entry *) NULL) &&
      ((h->type == bfd_link_hash_defined) || (h->type == bfd_link_hash_defweak)))
    {
      /* symbol is defined, re-define it */
      if (pic30_debug)
        {
          printf("\nRe-defining symbol %s, section was %s, value was %lx\n",
                 name, h->u.def.section->name, h->u.def.value);
          printf("  New values: section %s, value %lx\n", sec->name, value);
        }
      h->u.def.section = sec;
      h->u.def.value = value;
    }
}

/*
** Undefine one symbol by BFD
**
** This routine is called by bfd_link_hash_traverse()
*/
static boolean
bfd_pic30_undefine_one_symbol_bfd (h, p)
     struct bfd_hash_entry *h;
     PTR p;
{
  struct bfd_link_hash_entry *H = (struct bfd_link_hash_entry *) h;
  struct traverse_hash_info *info = ( struct traverse_hash_info *) p;

  /* if this symbol is defined, and owned by the BFD in question */
  if ((( H->type == bfd_link_hash_defined) ||
       ( H->type == bfd_link_hash_defweak)) &&
      ( H->u.def.section->owner == info->abfd))

    /* mark it undefined */
    H->type = bfd_link_hash_undefined;
 
  /* traverse the entire hash table */
  return true;
} /* static boolean bfd_pic30_undefine_one_symbol_bfd (...)*/


/*
** Undefine all symbols owned by an bfd
*/
static void
bfd_pic30_undefine_symbols_bfd (target)
     bfd *target;
{
    struct traverse_hash_info info;
    info.abfd = target;

    /* traverse the global linker hash table */
    bfd_link_hash_traverse (link_info.hash,
			    bfd_pic30_undefine_one_symbol_bfd, &info);

} /* static void bfd_pic30_undefine_symbols_bfd (...) */


/*
** Remove an archive module
*/
static void
bfd_pic30_remove_archive_module (name)
     const char *const name;
{
  asection *sec;
  lang_input_statement_type *prev = (lang_input_statement_type *) 0;

  LANG_FOR_EACH_INPUT_STATEMENT (is)
    {
      if (strcmp (is->the_bfd->filename, name) == 0)
        {
          if (pic30_debug)
            {
              printf("\nRemoving %s\n", name);
              printf("  %s had %d symbols\n", name , is->the_bfd->symcount);
            }

          /* remove the symbols */
          bfd_pic30_undefine_symbols_bfd (is->the_bfd);
          is->the_bfd->symcount = 0;

          /* remove the sections */
          sec = is->the_bfd->sections;
          if (sec == 0)
            break;

          /* loop through the sections in this bfd */
          for (; sec != 0; sec = sec->next)
            {
              sec->flags = SEC_NEVER_LOAD;
              sec->_raw_size = 0;
            }
          break;
        }
      prev = is;
    }
} /* static void bfd_pic30_remove_archive_module (.) */


/*
** Print memory region info
*/
static void
bfd_pic30_print_region_info (name)
     const char *const name;
{
  lang_memory_region_type *region;

  region = lang_memory_region_lookup (name);
  printf("  %s: start = %x, len = %x, current = %x\n", region->name,
         (unsigned int) region->origin,
         (unsigned int) region->length,
         (unsigned int) region->current);
  return;
}


/*****************************************************************************/

/*
** Emulation Routines
**
** These functions are called directly
** by entry points in the emulation transfer structure.
**
*/


/*
 * Emulation routine: bfd_pic30_before_parse
 *
 * - initialize data structures needed by the emulation
 *  
 */
static void
bfd_pic30_before_parse()
{
#if 0
  fprintf( stderr, "before_parse\n");
#endif
}


/*
 * Emulation routine: bfd_pic30_process_bfd_after_open
 *
 * This routine is called for each input bfd. It
 * loops through all the input sections and stores
 * handle relocations in a hash table. This table
 * is later used to construct the handle jump table.
 *
 * Sections named .comment have their ALLOC and LOAD
 * flags cleared.
 *
 * Also looks for an input file named sbrk.o. If found,
 * we know that a heap will be required at runtime.
 *
 */
static boolean
bfd_pic30_process_bfd_after_open (abfd, info)
     bfd *abfd;
     struct bfd_link_info *info ATTRIBUTE_UNUSED;
{
  asection *sec;
  arelent **p;
  const char *sym_name;
  struct handle_hash_entry * handle;
  boolean file_printed = false;

  /* check the input file name */
  if (!strcmp(abfd->filename, "sbrk.o"))
    {
      pic30_heap_required = 1;
    }

  /* here we have a bfd that is to be included on the link. We have a hook
     to do reloc rummaging, before section sizes are nailed down.  */

  sec = abfd->sections;
  if (sec == 0)
    {
      return true;
    }

  /* loop through the sections in this bfd */
  for (; sec != 0; sec = sec->next)
  {
    flagword old_flags = sec->flags;

    /* process input section flags here, if necessary */

    /* report if flags were changed */
    if ((pic30_debug) && (sec->flags != old_flags))
      printf("   adjust flags: %s %x\n", sec->name, sec->flags);

    if (sec->reloc_count == 0)
      continue;

    rels = bfd_pic30_load_relocs(abfd, sec);
    if (!rels)
      continue;

    /* loop through the relocs in this section */
    for (p = rels; relcount && *p != (arelent *) NULL; p++, relcount--)
      {
        arelent *q = *p;

        if ((q->howto->type == R_PIC30_WORD_HANDLE) ||
            (q->howto->type == R_PIC30_P_HANDLE) ||
            (q->howto->type == R_PIC30_HANDLE))
          {
#if 0
            DUMP_CANON_RELOC(q->howto->name, q);
#endif
            if (!pic30_handles) break;

            /* print file heading once */
            if ((pic30_debug) && !file_printed)
              {
                printf("%s:\n", abfd->filename);
                file_printed++;
              }

            /* make sure handle hash table is ready */
            if (!handles)
              handles = handle_hash_init();

            /* get the target symbol */
            if (q->sym_ptr_ptr && *q->sym_ptr_ptr)
              sym_name = (*(q->sym_ptr_ptr))->name;
            else
              sym_name = NULL;

            if (pic30_debug)
              printf("  Handle symbol: %s %x %x %x\n",
                     (*(q->sym_ptr_ptr))->name,
                     (*(q->sym_ptr_ptr))->flags,
                     (unsigned int) (*(q->sym_ptr_ptr))->section,
                     (unsigned int) (*(q->sym_ptr_ptr))->value);

            /* if target is local symbol, make a global one */
            if (((*(q->sym_ptr_ptr))->flags & BSF_LOCAL) == BSF_LOCAL) 
              {
                char * global_name = (char *) bfd_alloc ( output_bfd,
                                                          (strlen(abfd->filename) +
                                                           strlen(sym_name) + 2));
                if (!global_name)
                  {
                    fprintf( stderr, 
                             "Link Error: not enough memory for handle symbol\n");
                    abort();
                  }
                strcpy (global_name, abfd->filename);
                strcat (global_name, ":");
                strcat (global_name, sym_name);
                /* if we haven't processed this symbol yet */
                handle = handle_hash_lookup (handles, global_name, 0, 0);
                if (!handle)
                  {
                    /* export it */
                    if (pic30_debug)
                      {
                        printf ("  Handle is local symbol %s  %x\n", sym_name,
                                (unsigned int) (*(q->sym_ptr_ptr))->value);
                        printf ("  ..creating global symbol %s\n", global_name);
                      }
                    _bfd_generic_link_add_one_symbol (info, abfd, global_name,
                                                      BSF_GLOBAL,
                                                      (*(q->sym_ptr_ptr))->section,
                                                      (*(q->sym_ptr_ptr))->value,
                                                      global_name, 1, 0, 0);
                  }
                sym_name = global_name;
              }

            /* add it to the handle list */
            bfd_pic30_add_handle_sym (sym_name);
            handle_relocs++;

          }
      }

    if (rels)
      free(rels); /* free the reloc structure */
  }

  return true;
} /* static boolean bfd_pic30_process_bfd_after_open (...)*/


/*
** Emulation routine: bfd_pic30_adjust_psv_sections()
** 
** This function scans through all output sections
** looking for the first READONLY section that requires
** section blocking to avoid a PSV page boundary.
** If found, that section defines a threshold above
** which all loadable sections must be re-allocated.
**
** The re-allocation process involves three lists:
**
**  readonly_sections -- READONLY sections in order of decreasing size
**  code_sections -- All other sections in order of decreasing size
**  free_blocks -- Free memory blocks in order of increasing size
**
** During re-allocation, we must respect the memory
** alignment and blocking requirements of each section.
*  As load addresses (LMAs) are changed, virtual addresses
** (VMAs) must be updated. For READONLY sections, the
** VMA is a data space address in the range 0x8000 to 0xFFFE.
*/
static void
bfd_pic30_adjust_psv_sections()
{
  bfd_vma next_page;
  unsigned int opb = bfd_octets_per_byte (output_bfd);
  struct lma_adjust_info info;
  lang_memory_region_type *prog;
  bfd_size_type actual_size;
  struct pic30_section *s;
  struct pic30_memory *b;
  int reallocate = 0;

  /* build an ordered list of output sections */
  pic30_init_section_list (&pic30_section_list);
  bfd_map_over_sections (output_bfd, &pic30_build_section_list, NULL);

  if (pic30_debug)
    printf("\nChecking READONLY sections:\n");

  /* look for the first READONLY section that needs blocking */
  for (s = pic30_section_list; s != NULL; s = s->next)
    if (s->sec && (s->sec->flags & SEC_READONLY) && (s->sec->_raw_size > 0))
      {
        /*
        ** check maximum size for READONLY sections
        **
        ** Note: sec->_raw_size includes phantom bytes,
        **       divide by opb to get actual length
        */
        actual_size = s->sec->_raw_size / opb;
        if (actual_size > 0x8000)
          einfo(_("%P%X: Error: READONLY section %s exceeds 32K bytes"
                  " (actual size = %u).\n"),
                  s->sec->name, actual_size);

        next_page = (s->sec->lma & ~ 0x7FFF) + 0x8000;
        if ((s->sec->lma + actual_size) > next_page)
          {
            /* section crosses PSV page boundary, must re-allocate */
            if (s->sec->user_set_vma)
              {
                /* The user has specified a starting address, possibly
                   with an OVERLAY command.  So we don't move it.*/
                einfo(_("%P: Warning: READONLY section %s at 0x%v"
                        " crosses a PSVPAG boundary.\n"),
                      s->sec->name, s->sec->lma);
                break;
              }

            if (pic30_debug)
              printf("  section %s needs blocking at %lx\n",
                     s->sec->name, s->sec->lma);
            info.base = s->sec->lma;
            reallocate++;
            break;
          }
        else
          {
            /* section is OK where it is, just update the VMA */
            if (s->sec->user_set_vma == 0)
              s->sec->vma = PSV_BASE + (s->sec->lma & 0x7FFF);
            if (pic30_debug)
              printf("  section %s vma = %lx, lma = %lx %s\n",
                     s->sec->name, s->sec->vma, s->sec->lma,
                     s->sec->user_set_vma ? "(fixed)" : "");
          }

      }

  pic30_free_section_list(&pic30_section_list);

  /* exit if there's nothing else to do */
  if (!reallocate)
    return;

  /* determine the upper limit of available memory */
  prog = lang_memory_region_lookup ("program");
#define DEFAULT_PROG_LIMIT 0x80000
  if (prog->length == ~(bfd_size_type) 0) /* if no program region specified */
    {
      info.limit = DEFAULT_PROG_LIMIT;
      einfo(_("%P: Warning: program memory region not specified."
              " Using default upper limit of 0x%v\n"), info.limit);
    }
  else
    info.limit = prog->origin + prog->length;

  /* build a list of READONLY sections above the threshold */
  pic30_init_section_list (&readonly_sections);
  bfd_map_over_sections (output_bfd, &pic30_build_readonly_section_list, &info);

  /* build a list of CODE sections above the threshold */
  pic30_init_section_list (&code_sections);
  bfd_map_over_sections (output_bfd, &pic30_build_code_section_list, &info);

  if (pic30_debug)
    {
      printf("\nREADONLY sections:\n");
      for (s = readonly_sections; s != NULL; s = s->next)
        if (s->sec)
          {
            printf("  section %s %lx %lx %x %s\n",
                   s->sec->name, s->sec->lma,
                   s->sec->_raw_size/opb, s->sec->alignment_power,
                   s->sec->user_set_vma ? "(fixed)" : "");
          }
      printf("\nCODE sections:\n");
      for (s = code_sections; s != NULL; s = s->next)
        if (s->sec)
          {
            printf("  section %s %lx %lx %x %s\n",
                   s->sec->name, s->sec->lma,
                   s->sec->_raw_size/opb, s->sec->alignment_power,
                   s->sec->user_set_vma ? "(fixed)" : "");
          }

    }

  pic30_append_section_lists (readonly_sections, code_sections);

  /* build the Freelist: free memory blocks above the threshold */
  pic30_init_memory_list (&free_blocks);
  pic30_add_to_memory_list (free_blocks, info.base, (info.limit - info.base));

  if (pic30_debug)
    {
      printf("\nFree memory blocks:\n");
      for (b = free_blocks; b != NULL; b = b->next)
        if (b->size)
          printf("  block %lx %lx\n", b->addr, b->size);
    }

  if (pic30_debug)
    printf("\nRe-allocating program memory range %lx to %lx\n",
           info.base, info.limit);

  /* allocate static sections */
  for (s = readonly_sections; s != NULL; s = s->next)
    if ((s->sec) && (s->sec->user_set_vma))
      {
        b = pic30_static_assign_memory (free_blocks, s->sec->_raw_size/opb,
                                        s->sec->lma);
        if (!b)
          einfo(_("%P%X: Error: Could not allocate section %s at %lx\n"),
                s->sec->name, s->sec->lma);
        else
          {
            /* add new free block(s) for any remainder space */
            if (b->offset)
              pic30_add_to_memory_list (free_blocks, b->addr, b->offset);

            if ((b->offset + s->sec->_raw_size/opb) < b->size)
              {
                bfd_vma new_addr = b->addr + b->offset + s->sec->_raw_size/opb;
                pic30_add_to_memory_list (free_blocks, new_addr,
                                       b->size - b->offset - s->sec->_raw_size/opb);
              }
            /* remove the block we just used */
            pic30_remove_from_memory_list (free_blocks, b);
          }
      }

  /* allocate relocatable sections*/
  for (s = readonly_sections; s != NULL; s = s->next)
    if ((s->sec) && (s->sec->user_set_vma == 0))
      {
        b = pic30_best_fit_memory (free_blocks, s->sec->_raw_size/opb,
                                   s->sec->alignment_power,
                                   true);
        if (!b)
          einfo(_("%P%X: Error: Not enough contiguous memory"
                  " for section %s.\n"), s->sec->name);
        else
          {
            bfd_vma old_address = s->sec->lma;

            /* update the section header info */
            s->sec->lma = b->addr + b->offset;
            if (s->sec->flags & SEC_READONLY)
              s->sec->vma = PSV_BASE + (s->sec->lma & 0x7FFF);
            else
              s->sec->vma = s->sec->lma;

            if (pic30_debug)
              printf("  moving section %s from %lx to %lx\n",
                     s->sec->name, old_address, s->sec->lma);

            /* add new free block(s) for any remainder space */
            if (b->offset)
              pic30_add_to_memory_list (free_blocks, b->addr, b->offset);

            if ((b->offset + s->sec->_raw_size/opb) < b->size)
              {
                bfd_vma new_addr = b->addr + b->offset + s->sec->_raw_size/opb;
                pic30_add_to_memory_list (free_blocks, new_addr,
                                       b->size - b->offset - s->sec->_raw_size/opb);
              }
            /* remove the block we just used */
            pic30_remove_from_memory_list (free_blocks, b);
          }
      }

  if (pic30_debug)
    {
      printf("\nFree memory blocks:\n");
      for (b = free_blocks; b != NULL; b = b->next)
        if (b->size)
          printf("  block %lx %lx\n", b->addr, b->size);
    }

  pic30_free_section_list (&readonly_sections);   /* this frees both A & B lists */
  pic30_free_memory_list  (&free_blocks);

} /* static void bfd_pic30_adjust_psv_sections()*/


/*
** Emulation routine: bfd_pic30_tabulate_gaps()
** 
** - scan through all lang statements and
**    accumulate total gaps due to alignment
**    retrictions
**
** - the running total is stored in a global
**    symbol based on the section name.
*/
static void
bfd_pic30_tabulate_gaps_worker(statement)
     lang_statement_union_type *statement;
{
  if (statement->header.type == lang_padding_statement_enum)
    {
      char name[100];
      struct bfd_link_hash_entry *h = (struct bfd_link_hash_entry *) NULL;
      asection *sec = statement->padding_statement.output_section;
      unsigned int size = (unsigned int) statement->padding_statement.size;
      if (pic30_debug)
        printf("  section %s, gap = %d bytes\n", sec->name, size);

      /* look up or create a symbol to accumulate gap data */
      strncpy(name, sec->name, 95);
      strcat(name, "GAPS");
      h = bfd_link_hash_lookup (link_info.hash, name, false, false, true);

      if (h == (struct bfd_link_hash_entry *) NULL)
        /* create one */
        _bfd_generic_link_add_one_symbol (&link_info, output_bfd, name,
                                          BSF_WEAK, bfd_abs_section_ptr,
                                          size, name, 1, 0, 0);
      else
        /* accumulate gap size */
        h->u.def.value += size;
    }
}


static void
bfd_pic30_tabulate_gaps()
{
  if (pic30_debug)
    printf("\nLooking for alignment gaps:\n");
  lang_for_each_statement (bfd_pic30_tabulate_gaps_worker);
}


/*
** Emulation routine: bfd_pic30_finish
**
** - perform any remaining tasks before emulation is completed
**  
*/
static void
bfd_pic30_finish()
{
  asection *sec;
  unsigned int const_psvpage = 0;
  unsigned int const_length = 0;

  /* don't let ldmain() change .text section flags */
  config.text_read_only = false;

  /* remove output sections with size = 0 */
  pic30_strip_sections(output_bfd);

  /* adjust psv sections upward, if necessary */
  bfd_pic30_adjust_psv_sections();

  /* see if section .const remains (size > 0) */
  sec = bfd_get_section_by_name(output_bfd, ".const");
  if (sec)
    {
      has_const_section = true;
      const_psvpage = sec->lma >> 15;
      const_length  = sec->_raw_size;
    }

  /* mark _COMMENT section NEVER_LOAD, if it exists */
  sec = bfd_get_section_by_name(output_bfd, _COMMENT);
  if (sec)
    sec->flags |= SEC_NEVER_LOAD;

  /* tabulate gaps in output sections */
  bfd_pic30_tabulate_gaps();

  /*
  ** Check for overflow of X and Near data space
  **  using symbols created by the linker script
  */
  {
    struct bfd_link_hash_entry *h;
    if ((h = bfd_pic30_is_defined_global_symbol("__X_OVERFLOW")) &&
        (h->u.def.value > 0))
      {
        einfo("%P%X: Link Error: X data space has overflowed by %d bytes\n",
              h->u.def.value);
      }
    if ((h = bfd_pic30_is_defined_global_symbol("__NEAR_OVERFLOW")) &&
        (h->u.def.value > 0))
      {
        einfo("%P%X: Link Error: Near data space has overflowed by %d bytes\n",
              h->u.def.value);
      }
  }

  /*
  ** Check for conflicting HEAPSIZE, STACKSIZE definitions
  */
  {
    struct bfd_link_hash_entry *h;
    if ((h = bfd_pic30_is_defined_global_symbol("HEAPSIZE")))
      {
        /* HEAPSIZE is defined, check for conflict */
        if (pic30_has_heap_option && (h->u.def.value != pic30_heap_size))
          fprintf ( stderr, "Warning: --heap option overrides HEAPSIZE symbol\n");
        else
          {
            pic30_heap_size = h->u.def.value;
            pic30_has_heap_option = 1;
          }
      }

    if ((h = bfd_pic30_is_defined_global_symbol("STACKSIZE")))
      {
        /* STACKSIZE is defined, check for conflict */
        if (pic30_has_stack_option && (h->u.def.value != pic30_stack_size))
          fprintf ( stderr, "Warning: --stack option overrides STACKSIZE symbol\n");
        else
          pic30_stack_size = h->u.def.value;
      }
  }

  /*
  ** If heap is required, make sure one is specified
  */
  if (pic30_heap_required && !pic30_has_heap_option)
    {
      einfo("%P%X Error: A heap is required, but has not been specified\n");
    }
    
  /*
  ** Check for user-defined stack
  **
  ** note: two symbols( __SP_init, __SPLIM_init) must be defined
  */
  {
    struct bfd_link_hash_entry *h;
    if ((h = bfd_pic30_is_defined_global_symbol("__SP_init")))
      {
        user_defined_stack = true;
        stack_base = h->u.def.value;
      }
    if ((h = bfd_pic30_is_defined_global_symbol("__SPLIM_init")))
      stack_limit = h->u.def.value;
    else
      user_defined_stack = false;
  }

  /*
  ** Calculate heap and stack parameters, if necessary
  */
  {
    lang_memory_region_type *region = lang_memory_region_lookup ("data");

    /* determine the upper limit of memory */
    if (region->length == ~(bfd_size_type) 0) /* if no data region specified, */
      {
        mem_limit = 0xFFFE;                     /*  use all of pic30 data space */
        einfo(_("%P: Warning: data memory region not specified."
                " Using default upper limit of 0x%v\n"), mem_limit);
      }
    else
      mem_limit = region->origin + region->length - 2;

    /* adjust for PSV window, if known */
    if ((has_const_section) && (PSV_BASE < mem_limit))
      {
        if (pic30_debug)
          printf("\nAdjusting data memory limit from %x to %x\n",
                 mem_limit, (PSV_BASE - 2));

        mem_limit = PSV_BASE - 2;

        if (region->current > mem_limit)
          {
            einfo("%P%X Error: Data region overlaps PSV window (%d bytes)\n",
                  (int) (region->current -  mem_limit));
          }
      }

    /* allow for SPLIM guardband (8 bytes) */
    mem_limit -= 8;

    /* allocate heap */
    heap_base = (region->current + 1) & ~1;  /* must be even */
    heap_limit = heap_base + pic30_heap_size;

    /* allocate stack, if necessary */
    if (!user_defined_stack)
      {
        stack_base = (heap_limit + 1) & ~1;  /* must be even */
        stack_limit = (mem_limit + 1) & ~1;
      }

    /* report dynamic memory values */
    if (pic30_debug)
      {
        printf("\nCalculating dynamic memory:\n");
        printf("  heap base = %x\n", heap_base);
        printf("  heap limit = %x\n", heap_limit);
        if (!user_defined_stack)
          {
            printf("  stack base = %x\n", stack_base);
            printf("  stack limit = %x\n", stack_limit);
          }
        printf("  data memory limit = %x\n", mem_limit);
      }
  }

  /*
  ** Range check the heap
  */
  if (pic30_heap_size && (heap_limit > mem_limit))
    {
      einfo("%P%X Error: Not enough memory for heap (%d bytes available)\n",
            (int) ((mem_limit - pic30_stack_size) - heap_base + 2));
    }

  /*
  ** Range check the stack
  */
  if ( (int) (stack_limit - stack_base + 2) < (int) pic30_stack_size)
    {
      einfo("%P%X Error: Not enough memory for stack (%d bytes available)\n",
            (int) (stack_limit - stack_base + 2));
    }

  /*
  ** Set heap and stack symbols for the runtime library
  */
  if (pic30_debug)
    printf("\nCreating __heap = %x\n", heap_base);
  _bfd_generic_link_add_one_symbol (&link_info, output_bfd, "__heap",
                                    BSF_GLOBAL, bfd_abs_section_ptr,
                                    heap_base, "__heap", 1, 0, 0);
  if (pic30_debug)
    printf("Creating __eheap = %x\n", heap_limit);
  _bfd_generic_link_add_one_symbol (&link_info, output_bfd, "__eheap",
                                    BSF_GLOBAL, bfd_abs_section_ptr,
                                    heap_limit, "__eheap", 1, 0, 0);
  if (!user_defined_stack)
    {
      if (pic30_debug)
        printf("Creating __SP_init = %x\n", stack_base);
      _bfd_generic_link_add_one_symbol (&link_info, output_bfd, "__SP_init",
                                        BSF_GLOBAL, bfd_abs_section_ptr,
                                        stack_base, "__SP_init", 1, 0, 0);
      if (pic30_debug)
        printf("Creating __SPLIM_init = %x\n", stack_limit);
      _bfd_generic_link_add_one_symbol (&link_info, output_bfd, "__SPLIM_init",
                                        BSF_GLOBAL, bfd_abs_section_ptr,
                                        stack_limit, "__SPLIM_init", 1, 0, 0);
    }

  /*
  ** Set tblpage, tbloffset symbols for data init template
  **   so the C startup module can find it.
  */
  if (pic30_data_init)
    {
      sec = bfd_get_section_by_name(output_bfd, ".dinit");
      if (sec)
        {
          unsigned int tblpage   = sec->lma >> 16;
          unsigned int tbloffset = sec->lma & 0xFFFF;
          if (pic30_debug)
            printf("Creating __dinit_tblpage = %x\n", tblpage);
          _bfd_generic_link_add_one_symbol (&link_info, output_bfd,
                                            "__dinit_tblpage", BSF_GLOBAL,
                                            bfd_abs_section_ptr, tblpage, 
                                            "__dinit_tblpage", 1, 0, 0);
          if (pic30_debug)
            printf("Creating __dinit_tbloffset = %x\n", tbloffset);
          _bfd_generic_link_add_one_symbol (&link_info, output_bfd,
                                            "__dinit_tbloffset", BSF_GLOBAL,
                                            bfd_abs_section_ptr, tbloffset,
                                            "__dinit_tbloffset", 1, 0, 0);
        }
    }

  /*
  ** Set psvpage, length symbols for .const section
  **   so the C startup module can initialize PSV
  */
  if (pic30_debug)
    printf("Creating __const_psvpage = %x\n", const_psvpage);
  _bfd_generic_link_add_one_symbol (&link_info, output_bfd,
                                    "__const_psvpage", BSF_GLOBAL,
                                    bfd_abs_section_ptr, const_psvpage, 
                                    "__const_psvpage", 1, 0, 0);
  if (pic30_debug)
    printf("Creating __const_length = %x\n", const_length);
  _bfd_generic_link_add_one_symbol (&link_info, output_bfd,
                                    "__const_length", BSF_GLOBAL,
                                    bfd_abs_section_ptr, const_length,
                                    "__const_length", 1, 0, 0);

} /* static void bfd_pic30_finish () */


/*
** Utility routine: bfd_pic30_report_program_sections()
**
** - show memory usage of LOADable sections
*/ 
static void
bfd_pic30_report_program_sections (sect, fp) 
     asection *sect;
     FILE *fp;
{
  file_ptr start = sect->lma;
  bfd_size_type total = sect->_raw_size;
  bfd_size_type PCunits = total / 2;
  bfd_size_type actual = total * .75;

  /* report correct usage for EEDATA sections */
  if (start >= EEDATA_BASE)
    actual = total / 2;

  /* report LOADable sections */
  if ((sect->flags & SEC_LOAD))
    {
      fprintf( fp, "%-10s%#12lx%#20lx%#16lx  (%ld)\n", sect->name,
             start, PCunits, actual, actual);
      actual_prog_memory_used += (total * .75);
    }

  return;
} /* static void bfd_pic30_report_program_sections (...)*/


/*
** Utility routine: bfd_pic30_report_data_sections()
**
** - show memory usage of BSS and DATA sections
*/ 
static void
bfd_pic30_report_data_sections (sect, fp) 
     asection *sect;
     FILE *fp;
{
  file_ptr start = sect->vma;
  unsigned int opb = bfd_octets_per_byte (output_bfd);
  bfd_size_type total = sect->_raw_size / opb; /* ignore phantom bytes */

  /* report BSS and DATA sections */
  if (((sect->flags & (SEC_ALLOC | SEC_LOAD | SEC_DATA)) == SEC_ALLOC) ||
      (sect->flags & SEC_DATA))
    {
      unsigned int gaps;
      char name[100];
      struct bfd_link_hash_entry *h = (struct bfd_link_hash_entry *) NULL;

      strncpy(name, sect->name, 95);
      strcat(name, "GAPS");
      h = bfd_link_hash_lookup (link_info.hash, name, false, false, true);

      if (h == (struct bfd_link_hash_entry *) NULL)
        gaps = 0;
      else
        gaps = h->u.def.value / opb; /* ignore phantom bytes */


/*       fprintf( fp, "%-10s%#12lx%#33lx  (%ld)\n", sect->name, */
      fprintf( fp, "%-10s%#12lx%#20x%#16lx  (%ld)\n", sect->name,
             start, gaps, total, total);
      data_memory_used += total;
    }

  return;
} /* static void bfd_pic30_report_data_sections (...)*/


/*
** Utility routine: bfd_pic30_report_memory_usage
**
** - print a section chart to file *fp
**/
static void
bfd_pic30_report_memory_usage (fp)
     FILE *fp;
{
  bfd_size_type max_heap, max_stack;
  struct pic30_section *s;

  /* build an ordered list of output sections */
  pic30_init_section_list(&pic30_section_list);
  bfd_map_over_sections(output_bfd, &pic30_build_section_list, NULL);

  /* print code header */
  fprintf( fp, "\n\nProgram Memory Usage\n\n");
  fprintf( fp, "section        address   length (PC units)   length (bytes) (dec)\n");
  fprintf( fp, "-------        -------   -----------------   --------------------\n");

  /* report code sections */
  for (s = pic30_section_list; s != NULL; s = s->next)
    if (s->sec)
      bfd_pic30_report_program_sections (s->sec, fp);

  /* print code summary */
  fprintf( fp, "\n         Total program memory used (bytes):     %#10lx  (%ld)\n",
         actual_prog_memory_used, actual_prog_memory_used);

  /* print data header */
  fprintf( fp, "\n\nData Memory Usage\n\n");
  fprintf( fp, "section        address      alignment gaps    total length  (dec)\n");
  fprintf( fp, "-------        -------      --------------    -------------------\n");

  /* report data sections */
  for (s = pic30_section_list; s != NULL; s = s->next)
    if (s->sec)
      bfd_pic30_report_data_sections (s->sec, fp);

  /* print data summary */
  fprintf( fp, "\n            Total data memory used (bytes):     %#10lx  (%ld)\n",
         data_memory_used, data_memory_used);

  /* print dynamic header */
  fprintf( fp, "\n\nDynamic Memory Usage\n\n");
  fprintf( fp, "region         address                      maximum length  (dec)\n");
  fprintf( fp, "------         -------                      ---------------------\n");

  /* report dynamic regions */
  max_heap  = heap_limit - heap_base;
  max_stack = (stack_limit + 2) - stack_base;
  fprintf( fp, "%-10s%#12x%#36lx  (%ld)\n", "heap",
           heap_base, max_heap, max_heap);
  fprintf( fp, "%-10s%#12x%#36lx  (%ld)\n", "stack",
           stack_base, max_stack, max_stack);

  /* print dynamic summary */
  fprintf( fp, "\n            Maximum dynamic memory (bytes): %#14lx  (%ld)\n\n",
         (max_heap + max_stack), (max_heap + max_stack));

  /* free the output section list */
  pic30_free_section_list(&pic30_section_list);

} /* static void bfd_pic30_report_memory_usage (...)*/


/*****************************************************************************/

/*
** Entry points in the emulation transfer structure
*/


/*
** before_parse() is called before options are parsed.
*/
static void
gld${EMULATION_NAME}_before_parse()
{
#ifndef TARGET_			/* I.e., if not generic.  */
  ldfile_set_output_arch ("`echo ${ARCH}`");
#endif /* not TARGET_ */
  bfd_pic30_before_parse();
}


/*
** after_open() is called after all input BFDS
** have been created.
*/
static void
gld${EMULATION_NAME}_after_open()
{
  /*
  ** Merge full- and reduced-set I/O functions
  */
  if (pic30_smart_io)
    {
      int i = 0;

      if (pic30_debug)
        printf("\nMerging smart-io functions:\n");

      while ( function_pairs[i].full_set )

        {
          struct bfd_link_hash_entry *full, *reduced;
          char *func1, *func2;

          func1 = function_pairs[i].full_set;
          func2 = function_pairs[i].reduced_set;
          if (pic30_debug)
            printf("\nLooking for (%s, %s) ...", func1, func2);

          if ((full = bfd_pic30_is_defined_global_symbol (func1)) &&
              (reduced = bfd_pic30_is_defined_global_symbol (func2)))
            {
              if (pic30_debug)
                printf("Found\n\nRedirecting %s -> %s\n", func2, func1);

              /* redirect reduced_set function to full_set function */
              reduced->u.def.value = full->u.def.value;
              reduced->u.def.section = full->u.def.section;

              /* remove the reduced_set module */
              bfd_pic30_remove_archive_module (function_pairs[i].module_name);
            }
          else
            if (pic30_debug)
              printf("Not Found\n");
          i++;
        }
    }

  if (pic30_debug)
    printf("\nLooping through the input bfds:\n");
  {
    /* loop through all of the input bfds */
    LANG_FOR_EACH_INPUT_STATEMENT (is)
      {
        if (!bfd_pic30_process_bfd_after_open(is->the_bfd, &link_info))
          {
            einfo("Errors encountered processing file %s\n", is->filename);
          }
      }
  }
  /* OK, we've seen them all... now create the jump table */
  if (handle_relocs)
    {
      if (pic30_debug)
        { 
        printf("\nTotal Handle Relocations: %lu\n",handle_relocs);
        printf("Total Unique Handles: %u\n",handles->num);
        printf("Output BFD: %s\n",output_bfd->filename);
        }
      handle_bfd = bfd_pic30_create_jump_table_bfd (output_bfd);
      bfd_pic30_add_bfd_to_link (handle_bfd, handle_bfd->filename);
    }

  /*
  ** If data init support is enabled, create a BFD
  ** for section .dinit and add it to the link.
  */
  if (pic30_data_init)
    {
      init_bfd = bfd_pic30_create_data_init_bfd (output_bfd);
      bfd_pic30_add_bfd_to_link (init_bfd, init_bfd->filename);
    }

  /*
  ** Look for a user-defined __reset function.
  **
  ** If we find one, remove any that were loaded
  ** from archives.
  **
  ** Note: A user-defined __reset must be strongly defined.
  ** The archive versions must be weakly defined.
  */
  {
    struct bfd_link_hash_entry *h;
    if ((h = bfd_pic30_is_defined_global_symbol("__reset"))) /* implies strong */
      {
        if (pic30_debug)
            printf("\nFound a user-defined startup module\n");
        bfd_pic30_remove_archive_module ("crt0.o");
        bfd_pic30_remove_archive_module ("crt1.o");
        pic30_has_user_startup = 1;
      }
  }

  /* If the user has not provided a __reset function,
  ** load one of the standard modules.
  */
  if (pic30_has_user_startup == 0)
    {
      if (pic30_data_init)
        {
          /* 
          ** Set the definition of global symbol __reset to the
          ** primary startup module and discard the alternate module.
          */
          struct bfd_link_hash_entry *h;

          if (pic30_debug)
            printf("\nLooking for primary startup module..");

          if ((h = bfd_pic30_is_defined_global_symbol("__resetPRI")))
            {
              if (pic30_debug)
                printf("OK\n");
              bfd_pic30_redefine_global_symbol ("__reset",
                                                h->u.def.section, h->u.def.value);
              bfd_pic30_remove_archive_module ("crt1.o");
            }
          else
            {
              if (pic30_debug)
                printf("not found\n");
            }
        } /* if (pic30_data_init) */
      else
        {
          /* 
          ** Set the definition of global symbol __reset to the
          ** alternate startup module and discard the primary module.
          */
          struct bfd_link_hash_entry *h;

          if (pic30_debug)
              printf("\nLooking for alternate startup module..");

          if ((h = bfd_pic30_is_defined_global_symbol("__resetALT")))
            {
              if (pic30_debug)
                printf("OK\n");
              bfd_pic30_redefine_global_symbol ("__reset",
                                                h->u.def.section, h->u.def.value);
              bfd_pic30_remove_archive_module ("crt0.o");
            }
          else
            {
              if (pic30_debug)
                printf("not found\n");
            }
        } /* else (!pic30_data_init) */
    } /* if (pic30_has_user_startup == 0) */

  /*
  ** Look for a user-defined __DefaultInterrupt function.
  **
  ** If we don't find one, create it now.
  */
  {
    struct bfd_link_hash_entry *h;
    if ((h = bfd_pic30_is_defined_global_symbol("__DefaultInterrupt")))
      {
        if (pic30_debug)
            printf("\nFound a user-defined default interrupt handler\n");
        pic30_has_user_default_interrupt = 1;
      }
    else
      {
        if (pic30_debug)
            printf("\nCreating __DefaultInterrupt function\n");
        isr_bfd = bfd_pic30_create_default_isr_bfd (output_bfd);
        bfd_pic30_add_bfd_to_link (isr_bfd, isr_bfd->filename);
      }
  }

}/* static void gld${EMULATION_NAME}_after_open ()*/


/*
** before_allocation() is called after input sections
** have been mapped to output sections.
*/
static void  
gld${EMULATION_NAME}_before_allocation()
{
  if (pic30_debug)
    {
      printf("\nBefore allocation:\n");
      bfd_pic30_print_section_header();
      bfd_map_over_sections(output_bfd, &bfd_pic30_report_sections, 0);
    }
} /*static void gld${EMULATION_NAME}_before_allocation ()*/


/*
** after_allocation() is called after section
** offsets have been determined and output
** section sizes are known.
*/
static void  
gld${EMULATION_NAME}_after_allocation()
{
  int total_data = 0;
  asection *sec;

  {
    struct bfd_link_hash_entry *h;
    /*
    ** At this point, symbols declared in the
    ** linker script and on the command line
    ** are visible in the global hash table.
    **
    ** We look for a symbol called __NO_HANDLES
    ** and test its value. If true, remove the
    ** handle jump table.
    **
    ** This is a secondary method to suppress handles.
    ** The primary method is the --no-handles option.
    */
    if ((h = bfd_pic30_is_defined_global_symbol("__NO_HANDLES")))
      {
        if ((h->u.def.value) && handle_bfd)
          {
            sec = bfd_get_section_by_name(handle_bfd, ".handle");
            if (sec && (sec->_raw_size > 0))
              {
                sec->_raw_size = 0;
                bfd_set_section_size (handle_bfd, sec, 0);
                handles = 0; /* remove the hash table also...
                                bfd/coff-pic30.c tests this at reloc time
                                (note: BFD releases the memory) */
                if (pic30_debug)
                  printf("\nStripping section \".handle\"\n");
              }
          }
      }
  }

  if (pic30_data_init)
    {
      /* compute size of initialized data section */
      if (pic30_debug)
        printf("\nScanning data sections:\n");

      bfd_map_over_sections(output_bfd, &bfd_pic30_scan_data_sections, &total_data);
      total_data += 4; /* zero terminated */

      if (pic30_debug)
        {
          printf("  null terminator, template += 1 pword\n");
          printf("\nTotal initialized data %s: %x pwords\n",
                 pic30_pack_data ? "(packed)" : "(not packed)",
                 total_data / 4);
        }

      /* allocate memory for the template */
      init_data = (unsigned char *) bfd_alloc (output_bfd, total_data);
      if (!init_data)
        {
          fprintf( stderr, "Link Error: not enough memory for data template\n");
          abort();
        }

      /* attach it to the input section */
      sec = bfd_get_section_by_name(init_bfd, ".dinit");
      if (sec)
        {
          sec->_raw_size = total_data;
          sec->flags |= (SEC_HAS_CONTENTS | SEC_IN_MEMORY | SEC_LOAD | SEC_CODE);
          sec->contents = init_data;
          bfd_set_section_size (init_bfd, sec, total_data);
        }

      /* also update the output section */
      sec = bfd_get_section_by_name(output_bfd, ".dinit");
      if (sec)
        {
          sec->_raw_size = total_data;
          sec->flags |= (SEC_HAS_CONTENTS | SEC_IN_MEMORY | SEC_LOAD | SEC_CODE);
          sec->contents = init_data;
          bfd_set_section_size (output_bfd, sec, total_data);
        }
    } /* if (pic30_data_init) */
  else
    {
      if (pic30_debug)
        printf("\nScanning data sections:\n");

      bfd_map_over_sections(output_bfd, &bfd_pic30_skip_data_sections, NULL);
    } /* !pic30_data_init */


  if (pic30_debug)
    {
      printf("\nAfter allocation:\n");
      bfd_pic30_print_section_header();
      bfd_map_over_sections(output_bfd, &bfd_pic30_report_sections, 0);

      printf("\nMemory Regions\n");
      bfd_pic30_print_region_info("data");
      bfd_pic30_print_region_info("program");
    }
} /* static void gld${EMULATION_NAME}_after_allocation ()*/


/*
** finish() is called after symbol assignments are completed.
*/
static void  
gld${EMULATION_NAME}_finish()
{
  bfd_pic30_finish();
  if (pic30_debug)
    {
      printf("\nAfter finish:\n");
      bfd_pic30_print_section_header();
      bfd_map_over_sections(output_bfd, &bfd_pic30_report_sections, 0);

      printf("\nMemory Regions\n");
      bfd_pic30_print_region_info("data");
      bfd_pic30_print_region_info("program");

      printf("\nDynamic Memory\n");
      printf("  heap:  %x bytes, %x to %x\n",
             pic30_heap_size, heap_base, heap_limit);
      printf("  stack: %x bytes, %x to %x\n",
             (stack_limit - stack_base + 2), stack_base, stack_limit);
    } /* if (pic30_debug */

  if (config.map_file != NULL)
    bfd_pic30_report_memory_usage (config.map_file);
} /* static void gld${EMULATION_NAME}_finish ()*/


/*
** list_options() is called by the --help option.
*/
static void 
gld${EMULATION_NAME}_list_options (file) 
     FILE * file; 
{ 
  fprintf (file, _("  --data-init          Initialize data memory (default)\n")); 
  fprintf (file, _("  --no-data-init       Don't initialize data memory\n")); 
  fprintf (file, _("  -D,--debug           Produce linker debugging messages\n")); 
  fprintf (file, _("  --handles            Create jump table for"
                   " function pointers (default)\n")); 
  fprintf (file, _("  --no-handles         Don't create jump table for"
                   " function pointers\n")); 
  fprintf (file, _("  --heap SIZE          Create heap of SIZE bytes\n")); 
  fprintf (file, _("  --pack-data          Use upper byte of program memory"
                   " to store data (default)\n")); 
  fprintf (file, _("  --no-pack-data       Don't use upper byte of program memory"
                   " to store data\n")); 
  fprintf (file, _("  --smart-io           Merge I/O library functions (default)\n")); 
  fprintf (file, _("  --no-smart-io        Don't merge I/O library functions\n")); 
  fprintf (file, _("  --stack SIZE         Set minimum stack to SIZE bytes"
                   " (default=16)\n")); 
} /* static void gld${EMULATION_NAME}_list_options () */


/*
** parse_args() is called for each command line option.
*/
static int 
gld${EMULATION_NAME}_parse_args (argc, argv) 
     int     argc; 
     char ** argv; 
{ 
  int        longind; 
  int        optc; 
  int        prevoptind = optind; 
  int        prevopterr = opterr; 
  int        wanterror; 
  static int lastoptind = -1; 

  const char *data_init_option_err = "--data-init and --no-data-init";
  const char *handles_option_err   = "--handles and --no-handles";
  const char *pack_data_option_err = "--pack-data and --no-pack-data";
  const char *smart_io_option_err  = "--smart-io and --no-smart-io";
  const char *option_err = " options can not be used together\n";

  if (lastoptind != optind) 
    opterr = 0; 
  
  wanterror  = opterr; 
  lastoptind = optind; 
  optc   = getopt_long_only (argc, argv, shortopts, longopts, & longind); 
  opterr = prevopterr; 
  switch (optc) 
    { 
    default: 
      if (wanterror) 
        xexit (1); 
      optind =  prevoptind; 
      return 0;
    case 'D':
      pic30_debug = true;
      break;
    case HEAP_OPTION:
      pic30_has_heap_option = true;
      if (strstr(optarg, "0x") == 0)
        pic30_heap_size = atoi(optarg);
      else
        (void) sscanf(optarg, "%x", &pic30_heap_size);
      break;
    case DATA_INIT_OPTION:
      if (pic30_has_data_init_option && (!pic30_data_init))
        einfo(_("%P%F: Error: %s%s"), data_init_option_err, option_err);
      pic30_data_init = true;
      pic30_has_data_init_option = true;
      break;
    case NO_DATA_INIT_OPTION:
      if (pic30_has_data_init_option && (pic30_data_init))
        einfo(_("%P%F: Error: %s%s"), data_init_option_err, option_err);
      pic30_data_init = false;
      pic30_has_data_init_option = true;
      break;
    case HANDLES_OPTION:
      if (pic30_has_handles_option && (!pic30_handles))
        einfo(_("%P%F: Error: %s%s"), handles_option_err, option_err);
      pic30_handles = true;
      pic30_has_handles_option = true;
      break;
    case NO_HANDLES_OPTION:
      if (pic30_has_handles_option && (pic30_handles))
        einfo(_("%P%F: Error: %s%s"), handles_option_err, option_err);
      pic30_handles = false;
      pic30_has_handles_option = true;
      break;
    case PACK_DATA_OPTION:
      if (pic30_has_pack_data_option && (!pic30_pack_data))
        einfo(_("%P%F: Error: %s%s"), pack_data_option_err, option_err);
      pic30_pack_data = true;
      pic30_has_pack_data_option = true;
      break;
    case NO_PACK_DATA_OPTION:
      if (pic30_has_pack_data_option && (pic30_pack_data))
        einfo(_("%P%F: Error: %s%s"), pack_data_option_err, option_err);
      pic30_pack_data = false;
      pic30_has_pack_data_option = true;
      break;
    case STACK_OPTION:
      pic30_has_stack_option = true;
      if (strstr(optarg, "0x") == 0)
        pic30_stack_size = atoi(optarg);
      else
        (void) sscanf(optarg, "%x", &pic30_stack_size);
      break;
    case SMART_IO_OPTION:
      if (pic30_has_smart_io_option && (!pic30_smart_io))
        einfo(_("%P%F: Error: %s%s"), smart_io_option_err, option_err);
      pic30_smart_io = true;
      pic30_has_smart_io_option = true;
      break;
    case NO_SMART_IO_OPTION:
      if (pic30_has_smart_io_option && (pic30_smart_io))
        einfo(_("%P%F: Error: %s%s"), smart_io_option_err, option_err);
      pic30_smart_io = false;
      pic30_has_smart_io_option = true;
      break;
    } 
  
  return 1; 
} /* static int gld${EMULATION_NAME}_parse_args ()*/


/*
** get_script() returns the default linker script.
*/
static char *
gld${EMULATION_NAME}_get_script(isfile)
     int *isfile;

EOF

if test -n "$COMPILE_IN"
then
# Scripts compiled in.

# sed commands to quote an ld script as a C string.
sc="-f stringify.sed"

cat >>e${EMULATION_NAME}.c <<EOF
{			     
  *isfile = 0;

  if (link_info.relocateable == true && config.build_constructors == true)
    return
EOF
sed $sc ldscripts/${EMULATION_NAME}.xu                     >> e${EMULATION_NAME}.c
echo '  ; else if (link_info.relocateable == true) return' >> e${EMULATION_NAME}.c
sed $sc ldscripts/${EMULATION_NAME}.xr                     >> e${EMULATION_NAME}.c
echo '  ; else if (!config.text_read_only) return'         >> e${EMULATION_NAME}.c
sed $sc ldscripts/${EMULATION_NAME}.xbn                    >> e${EMULATION_NAME}.c
echo '  ; else if (!config.magic_demand_paged) return'     >> e${EMULATION_NAME}.c
sed $sc ldscripts/${EMULATION_NAME}.xn                     >> e${EMULATION_NAME}.c
echo '  ; else return'                                     >> e${EMULATION_NAME}.c
sed $sc ldscripts/${EMULATION_NAME}.x                      >> e${EMULATION_NAME}.c
echo '; }'                                                 >> e${EMULATION_NAME}.c

else
# Scripts read from the filesystem.

cat >>e${EMULATION_NAME}.c <<EOF
{			     
  *isfile = 1;

  if (link_info.relocateable == true && config.build_constructors == true)
    return "ldscripts/${EMULATION_NAME}.xu";
  else if (link_info.relocateable == true)
    return "ldscripts/${EMULATION_NAME}.xr";
  else if (!config.text_read_only)
    return "ldscripts/${EMULATION_NAME}.xbn";
  else if (!config.magic_demand_paged)
    return "ldscripts/${EMULATION_NAME}.xn";
  else
    return "ldscripts/${EMULATION_NAME}.x";
}
EOF

fi

cat >>e${EMULATION_NAME}.c <<EOF

struct ld_emulation_xfer_struct ld_${EMULATION_NAME}_emulation = 
{
  gld${EMULATION_NAME}_before_parse,
  syslib_default,
  hll_default,
  after_parse_default,
  gld${EMULATION_NAME}_after_open,
  /* after_allocation_default, */
  gld${EMULATION_NAME}_after_allocation,
  set_output_arch_default,
  ldemul_default_target,
  gld${EMULATION_NAME}_before_allocation,
  gld${EMULATION_NAME}_get_script,
  "${EMULATION_NAME}",
  "${OUTPUT_FORMAT}",
  /* NULL,	 finish */
  gld${EMULATION_NAME}_finish,
  NULL,	/* create output section statements */
  NULL,	/* open dynamic archive */
  NULL,	/* place orphan */
  NULL,	/* set symbols */
  gld${EMULATION_NAME}_parse_args,	/* parse args */
  NULL,	/* unrecognized file */
  gld${EMULATION_NAME}_list_options,	/* list options */
  NULL,	/* recognized file */
  NULL 	/* find_potential_libraries */
};
EOF
